/***************************************************************************
                          buttonwidget.h - eboxy button widget
                             -------------------
    begin                : Thu May 8 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INCLUDED__BUTTONWIDGET_H
#define INCLUDED__BUTTONWIDGET_H

using namespace std;

#include "widgets.h"
#include "actionobject.h"
   
class ButtonWidget: public ebWidget, ActionObject {
  public:
    ButtonWidget(const char *name, int x, int y, int w, int h);
    virtual ~ButtonWidget(void);
    virtual GUI_Widget *Instantiate(void);
    virtual void Uninstantiate(void);
    void SetNormalImage(ImageEntry *image);
    void SetSelectedImage(ImageEntry *image);
    void SetPressedImage(ImageEntry *image);
    void SetDisabledImage(ImageEntry *image);
    void SetFont(FontEntry *font);
    void SetFontColor(const SDL_Color *c);
    void SetFontFocusColor(const SDL_Color *c);
    void SetFontDisabledColor(const SDL_Color *c);
    void SetFontPressedColor(const SDL_Color *c);
    void SetCaption(const char *caption);
    void SetCaptionX(int value);
    void SetCaptionY(int value);
    void SetCaptionDropX(int value);
    void SetCaptionDropY(int value);
    void SetAction(const char *actionname);
    void SetEnabled(bool enabled);
    virtual void SetWidth(int value);
    virtual void SetHeight(int value);
    void SimulateClick(void);
    virtual void GotFocus(void);
    virtual void LostFocus(void);
    void Execute(void);

  private:
    char *caption;
    const char *action;
    GUI_Label *captionlabel;
    FontEntry *font;
    int fontsize;
    SDL_Color *fontcolor;
    SDL_Color *fontfocuscolor;
    SDL_Color *fontdisabledcolor;
    SDL_Color *fontpressedcolor;
    ImageEntry *normalimage;
    ImageEntry *selectedimage;
    ImageEntry *pressedimage;
    ImageEntry *disabledimage;
    int captionx;
    int captiony;
    int captiondropx;
    int captiondropy;
    bool enabled;
    void OnClick(GUI_Object *sender);
    void OnUpDown(GUI_Object *sender);
    void UpdateLabelColor(GUI_Widget *widget);
    void SetupCaptionLabel(GUI_Widget *rwidget);

    friend const char *ButtonWidget_GetCaption(ebCustomObject *sender);
    friend void ButtonWidget_SetCaption(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetFont(ebCustomObject *sender);
    friend void ButtonWidget_SetFont(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetFontSize(ebCustomObject *sender);
    friend void ButtonWidget_SetFontSize(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetFontColor(ebCustomObject *sender);
    friend void ButtonWidget_SetFontColor(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetFontFocusColor(ebCustomObject *sender);
    friend void ButtonWidget_SetFontFocusColor(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetFontPressedColor(ebCustomObject *sender);
    friend void ButtonWidget_SetFontPressedColor(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetFontDisabledColor(ebCustomObject *sender);
    friend void ButtonWidget_SetFontDisabledColor(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetImage(ebCustomObject *sender);
    friend void ButtonWidget_SetImage(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetSelectedImage(ebCustomObject *sender);
    friend void ButtonWidget_SetSelectedImage(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetPressedImage(ebCustomObject *sender);
    friend void ButtonWidget_SetPressedImage(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetDisabledImage(ebCustomObject *sender);
    friend void ButtonWidget_SetDisabledImage(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetCaptionX(ebCustomObject *sender);
    friend void ButtonWidget_SetCaptionX(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetCaptionY(ebCustomObject *sender);
    friend void ButtonWidget_SetCaptionY(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetCaptionDropX(ebCustomObject *sender);
    friend void ButtonWidget_SetCaptionDropX(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetCaptionDropY(ebCustomObject *sender);
    friend void ButtonWidget_SetCaptionDropY(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetEnabled(ebCustomObject *sender);
    friend void ButtonWidget_SetEnabled(ebCustomObject *sender, const char *value);
    friend const char *ButtonWidget_GetAction(ebCustomObject *sender);
    friend void ButtonWidget_SetAction(ebCustomObject *sender, const char *value);

    friend const char *ButtonWidget_Click(ebCustomObject *sender, const char *method, int numargs, const char *args[]);
};

#endif
