/***************************************************************************
           dynamicobject.h  -  parent object class supporting events
                             -------------------
    begin                : Thu May 22 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INCLUDED__DYNAMICOBJECT_H
#define INCLUDED__DYNAMICOBJECT_H

#include <vector>

#include "stringlist.h"
#include "object.h"
#include "methodhandler.h"
#include "externalmethodhandler.h"
#include "eventhandler.h"
#include "scriptobject.h"
#include "notifyobject.h"

using namespace std;

class ebDynamicObject: public ebObject {
  public:
    ebDynamicObject(const char *name);
    virtual ~ebDynamicObject(void);
    EventHandler *RegisterEventHandler(const char *event, ebObject *sink, int (*eventfunction)(const char *));
    EventHandler *RegisterEventHandler(const char *event, Script *script);
    ExternalMethodHandler *RegisterMethod(const char *method, ebObject *owner, int numargs, const char *(*methodfunc)(const char *, int, const char*[]));
    void UnregisterEventHandler(EventHandler *handler);
    int UnregisterEventHandler(const char *event, ebObject *sink);
    void UnregisterMethod(MethodHandler *handler);
    virtual const char *MethodCall(const char *method, int numargs, const char *args[]);;
    virtual bool EventHandled(const char *method);
    virtual const char *GetPropertyValue(const char *property);
    virtual void SetPropertyValue(const char *property, const char *value);
    virtual bool HasProperty(const char *property);
    virtual bool HasMethod(const char *method);
    MethodHandler *GetMethod(int index);
    int GetMethodCount(void);
    void ClearEventHandlers(void);
    void ClearMethodHandlers(void);
    bool IsFindable(void);
    void SetContainer(NotifyObject *container);
    void AddEvent(const char *eventname);
    int GetEventCount(void);
    const char *GetEvent(int index);
    const char *GetDefaultEvent(void);
  protected:
    vector<EventHandler *> eventhandlers;
    vector<MethodHandler *> methodhandlers;
    virtual int FireEvent(const char *event);
    virtual int FireEventWait(const char *event);
    bool findable;
    NotifyObject *container;
    StringList events;
    int defaultevent;
};

#endif
