/***************************************************************************
                 labelwidget.cpp  -  eboxy static label widget
                             -------------------
    begin                : Sat Aug 3 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cstring>
#include <cstdlib>
#include "SDL_gui.h"
#include "gui_extensions.h"

#include "utils.h"
#include "main.h"

#include "labelwidget.h"

using namespace std;

extern const int DEFAULT_FONTSIZE;
extern const SDL_Color DEFAULT_FONT_COLOR;
extern const char *TYPESTR_LABEL;

LabelWidget::LabelWidget(const char *name, int x, int y, int w, int h, const char *text, FontEntry *font)
: ebWidget(name, x, y) {
  this->w = w;
  this->h = h;
  this->text = strdup(text);
  this->font = font;
  this->fontsize = DEFAULT_FONTSIZE;
  this->wordwrap = false;
  this->autosize = false;
  this->align = WIDGET_VERT_TOP | WIDGET_HORIZ_LEFT;
  this->type = TYPESTR_LABEL;

  AddProperty("caption", LabelWidget_GetCaption, LabelWidget_SetCaption);
  AddProperty("font", LabelWidget_GetFont, LabelWidget_SetFont, true);
  AddProperty("fontsize", LabelWidget_GetFontSize, LabelWidget_SetFontSize);
  AddProperty("fontcolor", LabelWidget_GetFontColor, LabelWidget_SetFontColor);
  AddProperty("textalign", LabelWidget_GetTextAlign, LabelWidget_SetTextAlign);
  AddProperty("wordwrap", LabelWidget_GetWordWrap, LabelWidget_SetWordWrap);
  AddProperty("autosize", LabelWidget_GetAutoSize, LabelWidget_SetAutoSize);

  defaultproperty = FindProperty("caption");
}

LabelWidget::~LabelWidget(void) {
  if(text)
    free(text);
}

GUI_Widget *LabelWidget::Instantiate(void) {
  GUI_ExtLabel *label = new GUI_ExtLabel(name, x, y, w, h, font->font, text);
  label->SetTextColor((int)fontcolor.r, (int)fontcolor.g, (int)fontcolor.b);
  label->SetWordWrap(wordwrap);
  label->SetAutoSize(autosize);
  label->SetAlign(align);
  label->SetFlags(WIDGET_TRANSPARENT);
  realwidget = label;
  realwidget->MarkChanged(); // fixes bug with SDL_gui (doesn't draw if changed then added)
  return label;
}

void LabelWidget::SetText(const char *text) {
  if(this->text)
    free(this->text);
  this->text = strdup(text);
  if(realwidget) {
    GUI_Lock();
    SDL_Rect a = GetArea();
    realwidget->GetParent()->Erase(&a);
    ((GUI_ExtLabel *)realwidget)->SetText(this->text);
    this->InvalidateArea();
    GUI_Unlock();
  }
}

char *LabelWidget::GetText(void) {
  return text;
}

void LabelWidget::SetFont(FontEntry *font) {
  this->font = font;
  if(realwidget)
    ((GUI_ExtLabel *)realwidget)->SetFont(this->font->font);
}

void LabelWidget::SetFontColor(SDL_Color c) {
  this->fontcolor = c;
  if(realwidget) {
    ((GUI_ExtLabel *)realwidget)->SetTextColor((int)this->fontcolor.r, (int)this->fontcolor.g, (int)this->fontcolor.b);
    realwidget->MarkChanged();
  }
}

void LabelWidget::SetWordWrap(bool wordwrap) {
  this->wordwrap = wordwrap;
  if(realwidget)
    ((GUI_ExtLabel *)realwidget)->SetWordWrap(wordwrap);
}

void LabelWidget::SetAutoSize(bool autosize) {
  this->autosize = autosize;
  if(realwidget)
    ((GUI_ExtLabel *)realwidget)->SetAutoSize(this->autosize);
}

void LabelWidget::SetAlign(int align) {
  this->align = (align & WIDGET_ALIGN_MASK);
  if(realwidget)
    ((GUI_ExtLabel *)realwidget)->SetAlign(this->align);
}


/* Property get/set functions */

// -- caption --

const char *LabelWidget_GetCaption(ebCustomObject *sender) {
  return ((LabelWidget *)sender)->text;
}

void LabelWidget_SetCaption(ebCustomObject *sender, const char *value) {
  ((LabelWidget *)sender)->SetText(value);
}

// -- font --

const char *LabelWidget_GetFont(ebCustomObject *sender) {
  return ((LabelWidget *)sender)->font->filename;
}

void LabelWidget_SetFont(ebCustomObject *sender, const char *value) {
  LabelWidget *label = ((LabelWidget *)sender);

  FontEntry *font = loadFont((char *)value, label->fontsize);
  if(font)
    label->SetFont(font);
}

// -- fontsize --

const char *LabelWidget_GetFontSize(ebCustomObject *sender) {
  return inttostr(((LabelWidget *)sender)->fontsize);
}

void LabelWidget_SetFontSize(ebCustomObject *sender, const char *value) {
  LabelWidget *label = ((LabelWidget *)sender);
  int size = atoi(value);

  label->fontsize = size;
  if(label->font)
    LabelWidget_SetFont(label, label->font->filename);
}

// -- fontcolor --

const char *LabelWidget_GetFontColor(ebCustomObject *sender) {
  return colortostr( ((LabelWidget *)sender)->fontcolor );
}

void LabelWidget_SetFontColor(ebCustomObject *sender, const char *value) {
  ((LabelWidget *)sender)->SetFontColor(strtocolor((char *)value));
}

// -- textalign --

const char *LabelWidget_GetTextAlign(ebCustomObject *sender) {
  return textaligninttostr( ((LabelWidget *)sender)->align );
}

void LabelWidget_SetTextAlign(ebCustomObject *sender, const char *value) {
  ((LabelWidget *)sender)->SetAlign(textalignstrtoint(value));
}

// -- autosize --

const char *LabelWidget_GetAutoSize(ebCustomObject *sender) {
  return booltostr( ((LabelWidget *)sender)->autosize );
}

void LabelWidget_SetAutoSize(ebCustomObject *sender, const char *value) {
  ((LabelWidget *)sender)->SetAutoSize(strtobool(value));
}

// -- wordwrap --

const char *LabelWidget_GetWordWrap(ebCustomObject *sender) {
  return booltostr( ((LabelWidget *)sender)->wordwrap );
}

void LabelWidget_SetWordWrap(ebCustomObject *sender, const char *value) {
  ((LabelWidget *)sender)->SetWordWrap(strtobool(value));
}
