/***************************************************************************
                          listboxwidget.cpp  -  eboxy listbox widget
                             -------------------
    begin                : Thu Nov 7 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <cstring>
#include <cstdlib>
#include "SDL_gui.h"

#include "utils.h"
#include "gui_extensions.h"
#include "main.h"

#include "listboxwidget.h"

using namespace std;

extern const char *TYPESTR_LISTBOX;
extern const int DEFAULT_FONTSIZE;

ListBoxWidget::ListBoxWidget(const char *name, int x, int y, int w, int h, FontEntry *font, SDL_Color fontcolor)
: ebWidget(name, x, y) {
  this->w = w;
  this->h = h;
  this->font = font;
  this->fontsize = DEFAULT_FONTSIZE;
  this->fontcolor = fontcolor;

  // Set defaults
  tabstop = true;
  backgroundimage = NULL;
  backgroundfocusimage = NULL;
  backgroundstyle = IMAGESTYLE_NORMAL;
  selitembackgroundimage = NULL;
  selitembackgroundfocusimage = NULL;
  selitembackgroundstyle = IMAGESTYLE_NORMAL;
  scrollbarbackgroundimage = NULL;
  scrollbarbackgroundstyle = IMAGESTYLE_NORMAL;
  scrollknobimage = NULL;
  selitemfontcolor.r = 255;
  selitemfontcolor.g = 255;
  selitemfontcolor.b = 0;
  selitemfontcolor.unused = 255;
  scrollbarwidth = 16;
  borderx = 0;
  bordery = 0;
  itemborderx = 0;
  itembordery = 0;
  itemheight = -1; // automatic (text height)
  selectedindex = 0;
  scrollbarmode = AUTOPROPERTY_AUTO;
  this->type = TYPESTR_LISTBOX;

  AddProperty("font", ListBoxWidget_GetFont, ListBoxWidget_SetFont, true);
  AddProperty("fontsize", ListBoxWidget_GetFontSize, ListBoxWidget_SetFontSize);
  AddProperty("fontcolor", ListBoxWidget_GetFontColor, ListBoxWidget_SetFontColor);
  AddProperty("background", ListBoxWidget_GetBackground, ListBoxWidget_SetBackground);
  AddProperty("backgroundfocus", ListBoxWidget_GetBackgroundFocus, ListBoxWidget_SetBackgroundFocus);
  AddProperty("backgroundstyle", ListBoxWidget_GetBackgroundStyle, ListBoxWidget_SetBackgroundStyle);
  AddProperty("selectedindex", ListBoxWidget_GetSelectedIndex, ListBoxWidget_SetSelectedIndex);
  AddProperty("selectedtext", ListBoxWidget_GetSelectedText, NULL);
  AddProperty("count", ListBoxWidget_GetCount, NULL);
  AddProperty("selitembackground", ListBoxWidget_GetSelItemBackground, ListBoxWidget_SetSelItemBackground);
  AddProperty("selitembackgroundfocus", ListBoxWidget_GetSelItemBackgroundFocus, ListBoxWidget_SetSelItemBackgroundFocus);
  AddProperty("selitembackgroundstyle", ListBoxWidget_GetSelItemBackgroundStyle, ListBoxWidget_SetSelItemBackgroundStyle);
  AddProperty("selitemfontcolor", ListBoxWidget_GetSelItemFontColor, ListBoxWidget_SetSelItemFontColor);
  AddProperty("scrollbarbackground", ListBoxWidget_GetScrollBarBackground, ListBoxWidget_SetScrollBarBackground);
  AddProperty("scrollbarbackgroundstyle", ListBoxWidget_GetScrollBarBackgroundStyle, ListBoxWidget_SetScrollBarBackgroundStyle);
  AddProperty("scrollbarknob", ListBoxWidget_GetScrollBarKnob, ListBoxWidget_SetScrollBarKnob);
  AddProperty("scrollbarwidth", ListBoxWidget_GetScrollBarWidth, ListBoxWidget_SetScrollBarWidth);
  AddProperty("scrollbar", ListBoxWidget_GetScrollBar, ListBoxWidget_SetScrollBar);
  AddProperty("borderx", ListBoxWidget_GetBorderX, ListBoxWidget_SetBorderX);
  AddProperty("bordery", ListBoxWidget_GetBorderY, ListBoxWidget_SetBorderY);
  AddProperty("itemborderx", ListBoxWidget_GetItemBorderX, ListBoxWidget_SetItemBorderX);
  AddProperty("itembordery", ListBoxWidget_GetItemBorderY, ListBoxWidget_SetItemBorderY);
  AddProperty("itemheight", ListBoxWidget_GetItemHeight, ListBoxWidget_SetItemHeight);

  AddMethod("additem", 1, ListBoxWidget_AddItem);
  AddMethod("removeitem", 1, ListBoxWidget_RemoveItem);
  AddMethod("removelastitem", 0, ListBoxWidget_RemoveLastItem);
  AddMethod("clear", 0, ListBoxWidget_Clear);
  AddMethod("setitemtext", 2, ListBoxWidget_SetItemText);
  
  AddEvent("OnSelectionChange");
  AddEvent("OnEmpty");
  AddEvent("OnChoose");
  AddEvent("OnGotFocus");
  AddEvent("OnLostFocus");
}

ListBoxWidget::~ListBoxWidget(void) {
  while(listitems.size() > 0) {
    char *str = listitems.back();
    listitems.pop_back();
    free(str);
  }
}

GUI_Widget *ListBoxWidget::Instantiate(void) {
  if(w<0) w=0;
  if(h<0) h=0;
  
  GUI_ListBox *listbox = new GUI_ListBox(name, x, y, w, h, font->font, font->size);
  listbox->SetTextColor((int)fontcolor.r, (int)fontcolor.g, (int)fontcolor.b);
  listbox->SetSelectedItemTextColor((int)selitemfontcolor.r, (int)selitemfontcolor.g, (int)selitemfontcolor.b);
  listbox->SetScrollBarWidth(scrollbarwidth);
  listbox->SetBorderX(borderx);
  listbox->SetBorderY(bordery);
  listbox->SetItemBorderX(itemborderx);
  listbox->SetItemBorderY(itembordery);
  listbox->SetItemHeight(itemheight);

  if(backgroundimage)
    listbox->SetBackgroundImage(backgroundimage->surface);
  if(backgroundfocusimage)
    listbox->SetBackgroundFocusImage(backgroundfocusimage->surface);
  listbox->SetBackgroundStyle(backgroundstyle);
  listbox->SetScrollBarMode(scrollbarmode);
  if(selitembackgroundimage)
    listbox->SetSelectedItemImage(selitembackgroundimage->surface);
  if(selitembackgroundfocusimage)
    listbox->SetSelectedItemFocusImage(selitembackgroundfocusimage->surface);
  listbox->SetSelectedItemStyle(selitembackgroundstyle);
  if(scrollbarbackgroundimage)
    listbox->SetScrollBarBackgroundImage(scrollbarbackgroundimage->surface);
  listbox->SetScrollBarBackgroundStyle(scrollbarbackgroundstyle);
  if(scrollknobimage)
    listbox->SetScrollBarKnobImage(scrollknobimage->surface);

  for(int i=0; i<listitems.size(); i++) {
    listbox->AddItem(listitems[i]);
  }

  listbox->SetSelectedIndex(selectedindex);

  GUI_EventHandler<ListBoxWidget> *choosecb = new GUI_EventHandler<ListBoxWidget>(this, &ListBoxWidget::OnChoose);
  listbox->SetChooseCallback(choosecb);
	choosecb->DecRef();

  GUI_EventHandler<ListBoxWidget> *selcb = new GUI_EventHandler<ListBoxWidget>(this, &ListBoxWidget::OnSelectionChange);
  listbox->SetSelectionChangeCallback(selcb);
	selcb->DecRef();

  realwidget = listbox;
  return listbox;
}

void ListBoxWidget::Uninstantiate(void) {
  ebWidget::Uninstantiate();
}

void ListBoxWidget::Clone(ebWidget *newwidget) {
  ebWidget::Clone(newwidget);
  // Copy list items across
  for(int i=0; i<listitems.size(); i++) {
    ((ListBoxWidget *)newwidget)->AddItem(listitems[i]);
  }
}

void ListBoxWidget::SetBackgroundImage(ImageEntry *image) {
  this->backgroundimage = image;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetBackgroundImage(image->surface);
}

void ListBoxWidget::SetBackgroundFocusImage(ImageEntry *image) {
  this->backgroundfocusimage = image;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetBackgroundFocusImage(image->surface);
}

void ListBoxWidget::SetBackgroundStyle(int style) {
  if(style == IMAGESTYLE_NORMAL || style == IMAGESTYLE_TILED) {
    backgroundstyle = style;
    if(realwidget)
      ((GUI_ListBox *)realwidget)->SetBackgroundStyle(style);
  }
}

void ListBoxWidget::SetSelectedItemImage(ImageEntry *image) {
  this->selitembackgroundimage = image;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetSelectedItemImage(image->surface);
}

void ListBoxWidget::SetSelectedItemFocusImage(ImageEntry *image) {
  this->selitembackgroundfocusimage = image;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetSelectedItemFocusImage(image->surface);
}

void ListBoxWidget::SetScrollBarMode(int scrollbarmode) {
  this->scrollbarmode = scrollbarmode;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetScrollBarMode(scrollbarmode);
}

void ListBoxWidget::SetSelectedItemStyle(int style) {
  if(style == IMAGESTYLE_NORMAL || style == IMAGESTYLE_TILED) {
    selitembackgroundstyle = style;
    if(realwidget)
      ((GUI_ListBox *)realwidget)->SetSelectedItemStyle(style);
  }
}

void ListBoxWidget::SetScrollBarBackgroundImage(ImageEntry *image) {
  this->scrollbarbackgroundimage = image;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetScrollBarBackgroundImage(image->surface);
}

void ListBoxWidget::SetScrollBarBackgroundStyle(int style) {
  if(style == IMAGESTYLE_NORMAL || style == IMAGESTYLE_TILED) {
    scrollbarbackgroundstyle = style;
    if(realwidget)
      ((GUI_ListBox *)realwidget)->SetScrollBarBackgroundStyle(style);
  }
}

void ListBoxWidget::SetScrollBarKnobImage(ImageEntry *image) {
  this->scrollknobimage = image;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetScrollBarKnobImage(image->surface);
}

void ListBoxWidget::SetFont(FontEntry *font) {
  this->font = font;
  if(realwidget) {
    ((GUI_ListBox *)realwidget)->SetFont(this->font->font);
    realwidget->MarkChanged();
  }
}

void ListBoxWidget::SetFontColor(SDL_Color c) {
  this->fontcolor = c;
  if(realwidget) {
    ((GUI_ListBox *)realwidget)->SetTextColor(c.r, c.g, c.b);
    realwidget->MarkChanged();
  }
}

void ListBoxWidget::SetSelectedItemFontColor(SDL_Color c) {
  this->selitemfontcolor = c;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetSelectedItemTextColor(c.r, c.g, c.b);
}

void ListBoxWidget::SetScrollBarWidth(int width) {
  this->scrollbarwidth = width;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetScrollBarWidth(width);
}

void ListBoxWidget::SetBorderX(int pixels) {
  this->borderx = pixels;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetBorderX(pixels);
}

void ListBoxWidget::SetBorderY(int pixels) {
  this->bordery = pixels;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetBorderY(pixels);
}

void ListBoxWidget::SetItemBorderX(int pixels) {
  this->itemborderx = pixels;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetItemBorderX(pixels);
}

void ListBoxWidget::SetItemBorderY(int pixels) {
  this->itembordery = pixels;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetItemBorderY(pixels);
}

void ListBoxWidget::SetItemHeight(int pixels) {
  this->itemheight = pixels;
  if(realwidget)
    ((GUI_ListBox *)realwidget)->SetItemHeight(pixels);
}

void ListBoxWidget::SetSelectedIndex(int index) {
  if(index >= 0 && index < listitems.size()) {
    selectedindex = index;
    if(realwidget)
      ((GUI_ListBox *)realwidget)->SetSelectedIndex(index);
  }
}

void ListBoxWidget::SelectNext(int count) {
  if(selectedindex < listitems.size() - 1 && listitems.size() > 0) {
    // Select the next item
    if(selectedindex + count > listitems.size() - 1)
      SetSelectedIndex(listitems.size() - 1);
    else
      SetSelectedIndex(selectedindex + count);
  }
}

void ListBoxWidget::SelectNextPage(void) {
  SelectNext(((GUI_ListBox *)realwidget)->GetVisibleItems());
}

void ListBoxWidget::SelectPrevious(int count) {
  if(selectedindex > 0) {
    // Select the previous item
    if(selectedindex - count < 0)
      SetSelectedIndex(0);
    else
      SetSelectedIndex(selectedindex - count);
  }
}

void ListBoxWidget::SelectPreviousPage(void) {
  SelectPrevious(((GUI_ListBox *)realwidget)->GetVisibleItems());
}

int ListBoxWidget::GetSelectedIndex(void) {
  return selectedindex;
}

int ListBoxWidget::GetItemCount(void) {
  return listitems.size();
}

void ListBoxWidget::AddItem(const char *text) {
  char *itemtext = strdup(text);
  listitems.push_back(itemtext);
  if(realwidget)
    ((GUI_ListBox *)realwidget)->AddItem(text);
}

void ListBoxWidget::RemoveItem(int index) {
  if(index >= 0 && index < listitems.size()) {
    vector<char *>::iterator iter = listitems.begin() + index;
    char *item = listitems[index];
    listitems.erase(iter);
    free(item);
    if(realwidget)
      ((GUI_ListBox *)realwidget)->RemoveItem(index);
    if(listitems.size() == 0)
      FireEvent("OnEmpty");
  }
  else
    cerr << "RemoveItem: Listbox " << name << " has no item with index " << index << endl;
}

void ListBoxWidget::Clear(void) {
  int numitems = listitems.size();

  while(listitems.size() > 0) {
    char *str = listitems.back();
    listitems.pop_back();
    free(str);
  }

  if(realwidget)
    ((GUI_ListBox *)realwidget)->Clear();

  if(numitems>0)
    FireEvent("OnEmpty");
}

void ListBoxWidget::SetItemText(int index, const char *text) {
  if(index >= 0 && index < listitems.size()) {
    char *olditem = listitems[index];
    listitems[index] = strdup(text);
    free(olditem);
    if(realwidget)
      ((GUI_ListBox *)realwidget)->SetItemText(index, text);
  }
  else
    cerr << "SetItemText: Listbox " << name << " has no item with index " << index << endl;
}

void ListBoxWidget::ChooseSelection(void) {
  if(selectedindex >= 0 && listitems.size() > 0)
    this->FireEvent("OnChoose");
}

void ListBoxWidget::OnChoose(GUI_Object *sender) {
  // Fired from the real listbox widget
  this->ChooseSelection();
}

void ListBoxWidget::OnSelectionChange(GUI_Object *sender) {
  // Fired from the real listbox widget
  if(realwidget)
    selectedindex = ((GUI_ListBox *)realwidget)->GetSelectedIndex();
  this->FireEvent("OnSelectionChange");
}

void ListBoxWidget::GotFocus(void) {
  if(!focused) {
    this->FireEvent("OnGotFocus");
    ebWidget::GotFocus();
    ((GUI_ListBox *)realwidget)->MarkChanged();
  }
}

void ListBoxWidget::LostFocus(void) {
  if(focused) {
    this->FireEvent("OnLostFocus");
    ebWidget::LostFocus();
    ((GUI_ListBox *)realwidget)->MarkChanged();
  }
}


/* Property get/set functions */

// -- font --

const char *ListBoxWidget_GetFont(ebCustomObject *sender) {
  return ((ListBoxWidget *)sender)->font->filename;
}

void ListBoxWidget_SetFont(ebCustomObject *sender, const char *value) {
  ListBoxWidget *textfield = ((ListBoxWidget *)sender);

  FontEntry *font = loadFont((char *)value, textfield->fontsize);
  if(font)
    textfield->SetFont(font);
}

// -- fontsize --

const char *ListBoxWidget_GetFontSize(ebCustomObject *sender) {
  return inttostr(((ListBoxWidget *)sender)->fontsize);
}

void ListBoxWidget_SetFontSize(ebCustomObject *sender, const char *value) {
  ListBoxWidget *textfield = ((ListBoxWidget *)sender);
  int size = atoi(value);

  textfield->fontsize = size;
  if(textfield->font)
    ListBoxWidget_SetFont(textfield, textfield->font->filename);
}

// -- fontcolor --

const char *ListBoxWidget_GetFontColor(ebCustomObject *sender) {
  return colortostr( ((ListBoxWidget *)sender)->fontcolor );
}

void ListBoxWidget_SetFontColor(ebCustomObject *sender, const char *value) {
  ((ListBoxWidget *)sender)->SetFontColor(strtocolor((char *)value));
}

// -- background --

const char *ListBoxWidget_GetBackground(ebCustomObject *sender) {
  if( ((ListBoxWidget *)sender)->backgroundimage )
    return ((ListBoxWidget *)sender)->backgroundimage->filename;
  else
    return "";
}

void ListBoxWidget_SetBackground(ebCustomObject *sender, const char *value) {
  ImageEntry *image = loadImage(value);
  if(image)
    ((ListBoxWidget *)sender)->SetBackgroundImage(image);
}

// -- backgroundfocus --

const char *ListBoxWidget_GetBackgroundFocus(ebCustomObject *sender) {
  if( ((ListBoxWidget *)sender)->backgroundfocusimage )
    return ((ListBoxWidget *)sender)->backgroundfocusimage->filename;
  else
    return "";
}

void ListBoxWidget_SetBackgroundFocus(ebCustomObject *sender, const char *value) {
  ImageEntry *image = loadImage(value);
  if(image)
    ((ListBoxWidget *)sender)->SetBackgroundFocusImage(image);
}

// -- backgroundstyle --

const char *ListBoxWidget_GetBackgroundStyle(ebCustomObject *sender) {
  return styleinttostr( ((ListBoxWidget *)sender)->backgroundstyle );
}

void ListBoxWidget_SetBackgroundStyle(ebCustomObject *sender, const char *value) {
  ((ListBoxWidget *)sender)->SetBackgroundStyle(stylestrtoint(value));
}

// -- selectedindex --

const char *ListBoxWidget_GetSelectedIndex(ebCustomObject *sender) {
  return inttostr( ((ListBoxWidget *)sender)->GetSelectedIndex() );
}

void ListBoxWidget_SetSelectedIndex(ebCustomObject *sender, const char *value) {
  ((ListBoxWidget *)sender)->SetSelectedIndex(atoi(value));
}

// -- selectedtext --

const char *ListBoxWidget_GetSelectedText(ebCustomObject *sender) {
  ListBoxWidget *listbox = ((ListBoxWidget *)sender);
  
  if(listbox->listitems.size() > 0)
    return listbox->listitems[listbox->selectedindex];
  else
    return "";
}

// -- count --

const char *ListBoxWidget_GetCount(ebCustomObject *sender) {
  return inttostr( ((ListBoxWidget *)sender)->listitems.size() );
}

// -- selitembackground --

const char *ListBoxWidget_GetSelItemBackground(ebCustomObject *sender) {
  if( ((ListBoxWidget *)sender)->selitembackgroundimage )
    return ((ListBoxWidget *)sender)->selitembackgroundimage->filename;
  else
    return "";
}

void ListBoxWidget_SetSelItemBackground(ebCustomObject *sender, const char *value) {
  ImageEntry *image = loadImage(value);
  if(image)
    ((ListBoxWidget *)sender)->SetSelectedItemImage(image);
}

// -- selitembackgroundfocus --

const char *ListBoxWidget_GetSelItemBackgroundFocus(ebCustomObject *sender) {
  if( ((ListBoxWidget *)sender)->selitembackgroundfocusimage )
    return ((ListBoxWidget *)sender)->selitembackgroundfocusimage->filename;
  else
    return "";
}

void ListBoxWidget_SetSelItemBackgroundFocus(ebCustomObject *sender, const char *value) {
  ImageEntry *image = loadImage(value);
  if(image)
    ((ListBoxWidget *)sender)->SetSelectedItemFocusImage(image);
}

// -- selitembackgroundstyle --

const char *ListBoxWidget_GetSelItemBackgroundStyle(ebCustomObject *sender) {
  return styleinttostr( ((ListBoxWidget *)sender)->selitembackgroundstyle );
}

void ListBoxWidget_SetSelItemBackgroundStyle(ebCustomObject *sender, const char *value) {
  ((ListBoxWidget *)sender)->SetSelectedItemStyle(stylestrtoint(value));
}

// -- selitemfontcolor --

const char *ListBoxWidget_GetSelItemFontColor(ebCustomObject *sender) {
  return colortostr( ((ListBoxWidget *)sender)->selitemfontcolor );
}

void ListBoxWidget_SetSelItemFontColor(ebCustomObject *sender, const char *value) {
  ((ListBoxWidget *)sender)->SetSelectedItemFontColor(strtocolor((char *)value));
}

// -- scrollbarbackground --

const char *ListBoxWidget_GetScrollBarBackground(ebCustomObject *sender) {
  if( ((ListBoxWidget *)sender)->scrollbarbackgroundimage )
    return ((ListBoxWidget *)sender)->scrollbarbackgroundimage->filename;
  else
    return "";
}

void ListBoxWidget_SetScrollBarBackground(ebCustomObject *sender, const char *value) {
  ImageEntry *image = loadImage(value);
  if(image)
    ((ListBoxWidget *)sender)->SetScrollBarBackgroundImage(image);
}

// -- scrollbarbackgroundstyle --

const char *ListBoxWidget_GetScrollBarBackgroundStyle(ebCustomObject *sender) {
  return styleinttostr( ((ListBoxWidget *)sender)->scrollbarbackgroundstyle );
}

void ListBoxWidget_SetScrollBarBackgroundStyle(ebCustomObject *sender, const char *value) {
  ((ListBoxWidget *)sender)->SetScrollBarBackgroundStyle(stylestrtoint(value));
}

// -- scrollbarknob --

const char *ListBoxWidget_GetScrollBarKnob(ebCustomObject *sender) {
  if( ((ListBoxWidget *)sender)->scrollknobimage )
    return ((ListBoxWidget *)sender)->scrollknobimage->filename;
  else
    return "";
}

void ListBoxWidget_SetScrollBarKnob(ebCustomObject *sender, const char *value) {
  ImageEntry *image = loadImage(value);
  if(image)
    ((ListBoxWidget *)sender)->SetScrollBarKnobImage(image);
}

// -- scrollbarwidth --

const char *ListBoxWidget_GetScrollBarWidth(ebCustomObject *sender) {
  return inttostr( ((ListBoxWidget *)sender)->scrollbarwidth );
}

void ListBoxWidget_SetScrollBarWidth(ebCustomObject *sender, const char *value) {
  ((ListBoxWidget *)sender)->SetScrollBarWidth(atoi(value));
}

// -- scrollbar --

const char *ListBoxWidget_GetScrollBar(ebCustomObject *sender) {
  return autointtostr( ((ListBoxWidget *)sender)->scrollbarmode );
}

void ListBoxWidget_SetScrollBar(ebCustomObject *sender, const char *value) {
  ((ListBoxWidget *)sender)->SetScrollBarMode(autostrtoint(value));
}

// -- borderx --

const char *ListBoxWidget_GetBorderX(ebCustomObject *sender) {
  return inttostr( ((ListBoxWidget *)sender)->borderx );
}

void ListBoxWidget_SetBorderX(ebCustomObject *sender, const char *value) {
  ((ListBoxWidget *)sender)->SetBorderX(atoi(value));
}

// -- bordery --

const char *ListBoxWidget_GetBorderY(ebCustomObject *sender) {
  return inttostr( ((ListBoxWidget *)sender)->bordery );
}

void ListBoxWidget_SetBorderY(ebCustomObject *sender, const char *value) {
  ((ListBoxWidget *)sender)->SetBorderY(atoi(value));
}

// -- itemborderx --

const char *ListBoxWidget_GetItemBorderX(ebCustomObject *sender) {
  return inttostr( ((ListBoxWidget *)sender)->itemborderx );
}

void ListBoxWidget_SetItemBorderX(ebCustomObject *sender, const char *value) {
  ((ListBoxWidget *)sender)->SetItemBorderX(atoi(value));
}

// -- itembordery --

const char *ListBoxWidget_GetItemBorderY(ebCustomObject *sender) {
  return inttostr( ((ListBoxWidget *)sender)->itembordery );
}

void ListBoxWidget_SetItemBorderY(ebCustomObject *sender, const char *value) {
  ((ListBoxWidget *)sender)->SetItemBorderY(atoi(value));
}

// -- itemheight --

const char *ListBoxWidget_GetItemHeight(ebCustomObject *sender) {
  return inttostr( ((ListBoxWidget *)sender)->itemheight );
}

void ListBoxWidget_SetItemHeight(ebCustomObject *sender, const char *value) {
  ((ListBoxWidget *)sender)->SetItemHeight(atoi(value));
}

/* Method functions */

const char *ListBoxWidget_AddItem(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  ((ListBoxWidget *)sender)->AddItem(args[0]);
  return "";
}

const char *ListBoxWidget_RemoveItem(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  ((ListBoxWidget *)sender)->RemoveItem(atoi(args[0]));
  return "";
}

const char *ListBoxWidget_RemoveLastItem(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  ListBoxWidget *listbox = ((ListBoxWidget *)sender);
  
  listbox->RemoveItem(listbox->listitems.size()-1);
  return "";
}

const char *ListBoxWidget_Clear(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  ((ListBoxWidget *)sender)->Clear();
  return "";
}

const char *ListBoxWidget_SetItemText(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  ((ListBoxWidget *)sender)->SetItemText(atoi(args[0]), args[1]);
  return "";
}
