/***************************************************************************
                          listboxwidget.h  -  eboxy listbox widget
                             -------------------
    begin                : Tue May 13 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INCLUDED__LISTBOXWIDGET_H
#define INCLUDED__LISTBOXWIDGET_H

#include "widgets.h"

using namespace std;
   
class ListBoxWidget: public ebWidget {
  public:
    ListBoxWidget(const char *name, int x, int y, int w, int h, FontEntry *font, SDL_Color fontcolor);
    ~ListBoxWidget(void);
    GUI_Widget *Instantiate(void);
    void Uninstantiate(void);
    void Clone(ebWidget *newwidget);
    void SetFont(FontEntry *font);
    void SetFontColor(SDL_Color c);
    void SetSelectedItemFontColor(SDL_Color c);
    void SetBackgroundImage(ImageEntry *image);
    void SetBackgroundFocusImage(ImageEntry *image);
    void SetBackgroundStyle(int style);
    void SetSelectedItemImage(ImageEntry *image);
    void SetSelectedItemFocusImage(ImageEntry *image);
    void SetSelectedItemStyle(int style);
    void SetScrollBarMode(int scrollbarmode);
    void SetScrollBarBackgroundImage(ImageEntry *image);
    void SetScrollBarBackgroundStyle(int style);
    void SetScrollBarKnobImage(ImageEntry *image);
    void SetScrollBarWidth(int width);
    void SetBorderX(int pixels);
    void SetBorderY(int pixels);
    void SetItemBorderX(int pixels);
    void SetItemBorderY(int pixels);
    void SetItemHeight(int pixels);
    void SetSelectedIndex(int index);
    int GetSelectedIndex(void);
    int GetItemCount(void);
    void SelectNext(int count);
    void SelectNextPage(void);
    void SelectPrevious(int count);
    void SelectPreviousPage(void);
    void AddItem(const char *text);
    void RemoveItem(int index);
    void Clear(void);
    void SetItemText(int index, const char *text);
    void ChooseSelection(void);
    void GotFocus(void);
    void LostFocus(void);
  private:
    vector<char *> listitems;
    FontEntry *font;
    int fontsize;
    SDL_Color fontcolor;
    SDL_Color selitemfontcolor;
    ImageEntry *backgroundimage;
    ImageEntry *backgroundfocusimage;
    int backgroundstyle;
    ImageEntry *selitembackgroundimage;
    ImageEntry *selitembackgroundfocusimage;
    int selitembackgroundstyle;
    int scrollbarmode;
    ImageEntry *scrollbarbackgroundimage;
    int scrollbarbackgroundstyle;
    ImageEntry *scrollknobimage;
    int scrollbarwidth;
    int borderx;
    int bordery;
    int itemborderx;
    int itembordery;
    int itemheight;
    int selectedindex;
    void OnChoose(GUI_Object *sender);
    void OnSelectionChange(GUI_Object *sender);

    friend const char *ListBoxWidget_GetFont(ebCustomObject *sender);
    friend void ListBoxWidget_SetFont(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetFontSize(ebCustomObject *sender);
    friend void ListBoxWidget_SetFontSize(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetFontColor(ebCustomObject *sender);
    friend void ListBoxWidget_SetFontColor(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetBackground(ebCustomObject *sender);
    friend void ListBoxWidget_SetBackground(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetBackgroundFocus(ebCustomObject *sender);
    friend void ListBoxWidget_SetBackgroundFocus(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetBackgroundStyle(ebCustomObject *sender);
    friend void ListBoxWidget_SetBackgroundStyle(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetSelectedIndex(ebCustomObject *sender);
    friend void ListBoxWidget_SetSelectedIndex(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetSelectedText(ebCustomObject *sender);
    friend const char *ListBoxWidget_GetCount(ebCustomObject *sender);
    friend const char *ListBoxWidget_GetSelItemBackground(ebCustomObject *sender);
    friend void ListBoxWidget_SetSelItemBackground(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetSelItemBackgroundFocus(ebCustomObject *sender);
    friend void ListBoxWidget_SetSelItemBackgroundFocus(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetSelItemBackgroundStyle(ebCustomObject *sender);
    friend void ListBoxWidget_SetSelItemBackgroundStyle(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetSelItemFontColor(ebCustomObject *sender);
    friend void ListBoxWidget_SetSelItemFontColor(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetScrollBarBackground(ebCustomObject *sender);
    friend void ListBoxWidget_SetScrollBarBackground(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetScrollBarBackgroundStyle(ebCustomObject *sender);
    friend void ListBoxWidget_SetScrollBarBackgroundStyle(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetScrollBarKnob(ebCustomObject *sender);
    friend void ListBoxWidget_SetScrollBarKnob(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetScrollBarWidth(ebCustomObject *sender);
    friend void ListBoxWidget_SetScrollBarWidth(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetScrollBar(ebCustomObject *sender);
    friend void ListBoxWidget_SetScrollBar(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetBorderX(ebCustomObject *sender);
    friend void ListBoxWidget_SetBorderX(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetBorderY(ebCustomObject *sender);
    friend void ListBoxWidget_SetBorderY(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetItemBorderX(ebCustomObject *sender);
    friend void ListBoxWidget_SetItemBorderX(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetItemBorderY(ebCustomObject *sender);
    friend void ListBoxWidget_SetItemBorderY(ebCustomObject *sender, const char *value);
    friend const char *ListBoxWidget_GetItemHeight(ebCustomObject *sender);
    friend void ListBoxWidget_SetItemHeight(ebCustomObject *sender, const char *value);

    friend const char *ListBoxWidget_AddItem(ebCustomObject *sender, const char *method, int numargs, const char *args[]);
    friend const char *ListBoxWidget_RemoveItem(ebCustomObject *sender, const char *method, int numargs, const char *args[]);
    friend const char *ListBoxWidget_RemoveLastItem(ebCustomObject *sender, const char *method, int numargs, const char *args[]);
    friend const char *ListBoxWidget_Clear(ebCustomObject *sender, const char *method, int numargs, const char *args[]);
    friend const char *ListBoxWidget_SetItemText(ebCustomObject *sender, const char *method, int numargs, const char *args[]);
};

#endif
