/***************************************************************************
                          main.cpp  -  eboxy main source file
                             -------------------
    begin                : Mon Jun 10 23:07:59 NZST 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <cstdio>
#include <iostream>
#include <cstring>
#include <cstdlib>

#include <ctype.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "SDL_gui.h"
#include "SDL_thread.h"
#include "SDL.h"
#include "getopt.h"

#ifdef HAVE_LIRC
#include <lirc/lirc_client.h>
#endif

#include "widgets.h"
#include "buttonwidget.h"
#include "picturewidget.h"
#include "togglebuttonwidget.h"
#include "labelwidget.h"
#include "scrollbarwidget.h"
#include "textfieldwidget.h"
#include "listboxwidget.h"
#include "page.h"
#include "timer.h"
#include "systemobject.h"
#include "main.h"

#include "utils.h"
#include "constants.h"
#include "errors.h"
#include "gui_extensions.h"
#include "pluginmanager.h"
#include "pluginconstants.h"
#include "sdluserevents.h"
#include "skinparser.h"
#include "conf.h"
#include "actionmanager.h"
#include "processobject.h"

using namespace std;

// Global variables
PluginManager plugin_manager;
ActionManager action_manager;
vector<keybinding> keybindings;
//J SDL_Joystick *joystick;

#ifdef HAVE_LIRC
pthread_t lirc_thread;
bool disablelirc = false;
#endif
bool disableaudio = false;
bool nokeypassthrough = false;
bool nokeyrepeat = false;
bool showkeys = false;
pthread_mutex_t processmutex;
pthread_mutex_t eventmutex;
pthread_mutex_t switchmutex;
//X pthread_cond_t switchcond;
vector<ebPage *> pages;
vector<ebPage *> dynpages;    // runtime-created pages (note these are part of the pages vector also)
vector<ebWidget *> tempwidgets; // temp area for runtime-created widgets 
vector<Process *> processes;
vector<ImageEntry *> images;
vector<FontEntry *> fonts;
char *xmlFile = NULL;
ebPage *currentPage = NULL;
ebPage *loadedPage = NULL;
SystemObject *systemObject;
bool signalexit = false;
bool signalhide = false;
GUI_CardStack *cards;
ebWidget *garbage;
bool hidecursor = false;
Process *hideprocess = NULL;

// Function prototypes

#ifdef HAVE_LIRC
void *lirc_function(void *arg);
#endif
void *exec_function(void *arg);
bool checkName(const char *name);
GUI_Screen *initGUI(int width, int height, int depth, int opts);
void unloadObjects(bool unloadplugins);
void removeFinishedProcesses(void);
void SendKeyEvent(SDL_Event e);
void SendTextInput(char c);
void nextItem(void);
void previousItem(void);
void nextItemPage(void);
void previousItemPage(void);
void firstItem(void);
void lastItem(void);
int switchPage(const char *pagename);
int switchFile(const char *filename, const char *pagename);

// Function implementations

int switchPage(const char *pagename) {
  ebPage *page = NULL;
  ebPage *oldpage;
	GUI_Screen *screen = GUI_GetScreen();

  // Find requested page
  // FIXME: code duplication w/pluginmanager
  for(int i=0;i<pages.size(); i++) {
    if(!strcasecmp(pages[i]->GetName(), pagename)) {
      page = pages[i];
    }
  }
  if(page) {
    // Switch current page to requested page

    // Tell loaded plugins we're about to change the page
    if(currentPage)
      plugin_manager.BroadcastMessage(PLMSG_BEFOREPAGECHANGE, (void *)pagename, NULL);
    else
      plugin_manager.BroadcastMessage(PLMSG_BEFOREFIRSTPAGELOAD, (void *)pagename, NULL);

    page->SetWidth(screen->GetWidth());
    page->SetHeight(screen->GetHeight());
    loadedPage = page;
    GUI_Widget *panel = page->Instantiate();
    cards->AddWidget(panel);

    if(currentPage) {
      cards->Next();
      oldpage = currentPage;
      cards->RemoveWidget(oldpage->GetInstance());
      garbage = oldpage;
    }
    currentPage = page;

    // Tell loaded plugins we're done changing the page
    plugin_manager.BroadcastMessage(PLMSG_AFTERPAGECHANGE, (void *)pagename, NULL);

    // Fire the OnShow event of the page
    currentPage->OnShow();
  }
  else {
    cerr << "Unable to find page \'" << pagename << "\'" << endl;
    return EBERR_OBJECT_NOT_FOUND;
  }

  return EBERR_SUCCESS;
}

int switchFile(const char *filename, const char *pagename) {
  char *fullpath;
  int rc = 0;

  if(!checkFileExists(filename))
    return EBERR_FILE_ERROR;

  // Tell loaded plugins we're about to load a new file
  plugin_manager.BroadcastMessage(PLMSG_BEFOREFILELOAD, (void *)filename, NULL);

  // Clear out current stuff, if any
  unloadObjects(false);

  fullpath = expandPath(filename);
  parseSkin(fullpath);

  if(pagename)
    rc = switchPage(pagename);
  if(!pagename || rc != EBERR_SUCCESS)
    switchPage(pages[0]->GetName());

  xmlFile = fullpath;

  // Tell loaded plugins we're done loading a new file
  plugin_manager.BroadcastMessage(PLMSG_AFTERFILELOAD, (void *)filename, NULL);

  // Tidy up mess
  if(garbage) {
    garbage->Uninstantiate();
    delete garbage;
    garbage = NULL;
  }

  return EBERR_SUCCESS;
}

// Unload all objects apart from current page (if any)
void unloadObjects(bool unloadplugins) {
  if(unloadplugins) {
    plugin_manager.UnloadAllPlugins();
    plugin_manager.UnloadAllCustomObjects();
  }
  // Clear out pages
  // FIXME: need to arrange for dynamic page persistence over file change
  while(pages.size() > 0) {
    if(pages.back() != currentPage /*&& (unloadplugins || plugin_manager.FindDynamicPage(pages.back()) == -1)*/) {
      pages.back()->Uninstantiate();
      delete pages.back();
    }
    pages.pop_back();
  }
  while(dynpages.size() > 0) {
    dynpages.pop_back();
  }
//X  if(plugin_manager.FindDynamicPage(currentpage) == -1)
    garbage = currentPage; // so we can unload it later
  // Clear out images
  while(images.size() > 0) {
    ImageEntry *image = images.back();
    image->surface->DecRef();
    free(image->filename);
    images.pop_back();
    delete image;
  }
  // Clear out fonts
  while(fonts.size() > 0) {
    FontEntry *font = fonts.back();
    font->font->DecRef();
    free(font->filename);
    fonts.pop_back();
    delete font;
  }
  systemObject->ClearEventHandlers();
  systemObject->ClearObjects();
  clearTemplates();
}

void SendKeyEvent(SDL_Event e) {
  ebWidget *widget = currentPage->GetFocusedWidget();

  if(widget != NULL) {
    if(strcmp(widget->GetType(), TYPESTR_TEXTFIELD) == 0) {
      TextFieldWidget *textfield = (TextFieldWidget *)widget;
      textfield->SendEvent(e);
    }
  }
}

void SendTextInput(char c) {
  ebWidget *widget = currentPage->GetFocusedWidget();

  if(widget != NULL) {
    if(strcmp(widget->GetType(), TYPESTR_TEXTFIELD) == 0) {
      TextFieldWidget *textfield = (TextFieldWidget *)widget;
      textfield->SendChar(c);
    }
  }
}

void nextWidget(void) {
  currentPage->FocusNextWidget();
}

void prevWidget(void) {
  currentPage->FocusPreviousWidget();
}

void acceptSelection(void) {
  ebWidget *widget = currentPage->GetFocusedWidget();

  if(widget != NULL) {
    if(strcmp(widget->GetType(), TYPESTR_BUTTON) == 0) {
      ButtonWidget *button = (ButtonWidget *)widget;
      button->SimulateClick();
    }
    else if(strcmp(widget->GetType(), TYPESTR_LISTBOX) == 0) {
      ListBoxWidget *listbox = (ListBoxWidget *)widget;
      listbox->ChooseSelection();
    }
    else if(strcmp(widget->GetType(), TYPESTR_TEXTFIELD) == 0) {
      // Set focus to next widget for textfields
      currentPage->FocusNextWidget();
    }
    else if(strcmp(widget->GetType(), TYPESTR_TOGGLEBUTTON) == 0) {
      ToggleButtonWidget *tbutton = (ToggleButtonWidget *)widget;
      tbutton->SetChecked(!tbutton->GetChecked());
    }
  }
}

void nextItem(void) {
  ebWidget *widget = currentPage->GetFocusedWidget();

  if(widget != NULL) {
    if(strcmp(widget->GetType(), TYPESTR_TEXTFIELD) == 0) {
      TextFieldWidget *textfield = (TextFieldWidget *)widget;
      textfield->SetCursorPos(textfield->GetCursorPos() + 1);
    }
    else if(strcmp(widget->GetType(), TYPESTR_LISTBOX) == 0) {
      ListBoxWidget *listbox = (ListBoxWidget *)widget;
      listbox->SelectNext(1);
    }
    else if(strcmp(widget->GetType(), TYPESTR_SCROLLBAR) == 0) {
      ScrollBarWidget *scrollbar = (ScrollBarWidget *)widget;
      scrollbar->Increment();
    }
    else if(strcmp(widget->GetType(), TYPESTR_TOGGLEBUTTON) == 0) {
      ToggleButtonWidget *tbutton = (ToggleButtonWidget *)widget;
      tbutton->SetChecked(true);
    }
  }
}

void previousItem(void) {
  ebWidget *widget = currentPage->GetFocusedWidget();

  if(widget != NULL) {
    if(strcmp(widget->GetType(), TYPESTR_TEXTFIELD) == 0) {
      TextFieldWidget *textfield = (TextFieldWidget *)widget;
      textfield->SetCursorPos(textfield->GetCursorPos() - 1);
    }
    else if(strcmp(widget->GetType(), TYPESTR_LISTBOX) == 0) {
      ListBoxWidget *listbox = (ListBoxWidget *)widget;
      listbox->SelectPrevious(1);
    }
    else if(strcmp(widget->GetType(), TYPESTR_SCROLLBAR) == 0) {
      ScrollBarWidget *scrollbar = (ScrollBarWidget *)widget;
      scrollbar->Decrement();
    }
    else if(strcmp(widget->GetType(), TYPESTR_TOGGLEBUTTON) == 0) {
      ToggleButtonWidget *tbutton = (ToggleButtonWidget *)widget;
      tbutton->SetChecked(false);
    }
  }
}

void nextItemPage(void) {
  ebWidget *widget = currentPage->GetFocusedWidget();

  if(widget != NULL) {
    if(strcmp(widget->GetType(), TYPESTR_LISTBOX) == 0) {
      ListBoxWidget *listbox = (ListBoxWidget *)widget;
      listbox->SelectNextPage();
    }
    else if(strcmp(widget->GetType(), TYPESTR_SCROLLBAR) == 0) {
      ScrollBarWidget *scrollbar = (ScrollBarWidget *)widget;
      scrollbar->IncrementPage();
    }
  }
}

void previousItemPage(void) {
  ebWidget *widget = currentPage->GetFocusedWidget();

  if(widget != NULL) {
    if(strcmp(widget->GetType(), TYPESTR_LISTBOX) == 0) {
      ListBoxWidget *listbox = (ListBoxWidget *)widget;
      listbox->SelectPreviousPage();
    }
    else if(strcmp(widget->GetType(), TYPESTR_SCROLLBAR) == 0) {
      ScrollBarWidget *scrollbar = (ScrollBarWidget *)widget;
      scrollbar->DecrementPage();
    }
  }
}

void firstItem(void) {
  ebWidget *widget = currentPage->GetFocusedWidget();

  if(widget != NULL) {
    if(strcmp(widget->GetType(), TYPESTR_TEXTFIELD) == 0) {
      TextFieldWidget *textfield = (TextFieldWidget *)widget;
      textfield->SetCursorPos(0);
    }
    else if(strcmp(widget->GetType(), TYPESTR_LISTBOX) == 0) {
      ListBoxWidget *listbox = (ListBoxWidget *)widget;
      listbox->SetSelectedIndex(0);
    }
    else if(strcmp(widget->GetType(), TYPESTR_SCROLLBAR) == 0) {
      ScrollBarWidget *scrollbar = (ScrollBarWidget *)widget;
      scrollbar->GotoStart();
    }
  }
}

void lastItem(void) {
  ebWidget *widget = currentPage->GetFocusedWidget();

  if(widget != NULL) {
    if(strcmp(widget->GetType(), TYPESTR_TEXTFIELD) == 0) {
      TextFieldWidget *textfield = (TextFieldWidget *)widget;
      textfield->SetCursorPos(strlen(textfield->GetText()));
    }
    else if(strcmp(widget->GetType(), TYPESTR_LISTBOX) == 0) {
      ListBoxWidget *listbox = (ListBoxWidget *)widget;
      listbox->SetSelectedIndex(listbox->GetItemCount()-1);
    }
    else if(strcmp(widget->GetType(), TYPESTR_SCROLLBAR) == 0) {
      ScrollBarWidget *scrollbar = (ScrollBarWidget *)widget;
      scrollbar->GotoEnd();
    }
  }
}

// These are the thread-safe versions of the above

void nextWidgetEvent(void) {
  pushUserEvent(EBSE_FOCUSNEXT, NULL, NULL);
}

void prevWidgetEvent(void) {
  pushUserEvent(EBSE_FOCUSPREVIOUS, NULL, NULL);
}

void acceptSelectionEvent(void) {
  pushUserEvent(EBSE_SELECTFOCUSED, NULL, NULL);
}

int switchFileEvent(const char *filename, const char *pagename) {
  char *pagename1 = NULL;
  if(pagename)
    pagename1 = strdup(pagename);
  pushUserEvent(EBSE_LOADFILE, strdup(filename), pagename1);

  // FIXME: this doesn't work
//X  pthread_cond_wait(&switchcond, &switchmutex);
  
  return 0; // FIXME: need to make this synchronous
}

int switchPageEvent(const char *pagename) {
  pushUserEvent(EBSE_SWITCHPAGE, strdup(pagename), NULL);

  // FIXME: this doesn't work
//X  pthread_cond_wait(&switchcond, &switchmutex);

  return 0; // FIXME: need to make this synchronous
}

int main(int argc, char **argv) {
	
  // Long-only command-line options - these must be outside the range of char (ie, >255)
  //   so that they don't collide with the short option characters
  enum {
    CLOPT_BPP = 256,
    CLOPT_VERSION,
    CLOPT_HELP,
    CLOPT_NOAUDIO,
    CLOPT_NOLIRC,
    CLOPT_NOKEYPT,
    CLOPT_NOKEYREPEAT,
    CLOPT_SHOWKEYS
  };

  char *lastpagename = NULL;

  int v = SDL_HWSURFACE;
	int w = 640;
	int h = 480;
	int d = 16;

  // Parse command line using getopt_long
  while(true) {
    static struct option long_options[] = {
      {"window",     no_argument,       0, 'w'},
      {"fullscreen", no_argument,       0, 'f'},
      {"hidecursor", no_argument,       0, 'h'},
      {"width",      required_argument, 0, 'x'},
      {"height",     required_argument, 0, 'y'},
      {"bpp",        required_argument, 0, CLOPT_BPP},
      {"showkeys",   no_argument,       0, CLOPT_SHOWKEYS},
      {"version",    no_argument,       0, CLOPT_VERSION},
      {"noaudio",    no_argument,       0, CLOPT_NOAUDIO},
      {"nokeypassthrough",    no_argument,       0, CLOPT_NOKEYPT},
      {"nokeyrepeat",no_argument,       0, CLOPT_NOKEYREPEAT},
#ifdef HAVE_LIRC
      {"nolirc",     no_argument,       0, CLOPT_NOLIRC},
#endif
      {"help",       no_argument,       0, CLOPT_HELP},
      {0, 0, 0, 0}
    };

    // getopt_long stores the option index here
    int option_index = 0;
    opterr = 1;

    int c = getopt_long(argc, argv, "wfx:y:h", long_options, &option_index);

    // Detect the end of the options
    if (c == -1)
      break;

    switch (c) {
      case 0:
        // Flags (unused currently, although they probably should be)
        break;

      case 'w':
  			v &= ~SDL_FULLSCREEN;
        break;

      case 'f':
  			v |= SDL_FULLSCREEN;
        break;

      case 'x':
  			w = atoi(optarg);
        break;

      case 'y':
  			h = atoi(optarg);
        break;

      case 'h':
        hidecursor = true;
        break;

      case CLOPT_HELP:
        cout << "eboxy " << APP_MAJOR_VERSION << "." << APP_MINOR_VERSION << "." << APP_SUB_VERSION << " by Paul Eggleton (bluelightning@bluelightning.org)" << endl;
        cout << endl;
        cout << "  Syntax: eboxy [xmlfile] [options]" << endl;
        cout << "  Options:" << endl;
        cout << "    -w  --window            Use windowed display (default)" << endl;
        cout << "    -f  --fullscreen        Use full-screen display" << endl;
        cout << "    -x  --width             Set width of display" << endl;
        cout << "    -y  --height            Set height of display" << endl;
        cout << "    -h  --hidecursor        Hide the mouse cursor" << endl;
        cout << "        --showkeys          Show keyboard key info on key presses" << endl;
        cout << "        --noaudio           Do not initialise audio" << endl;
        cout << "        --nokeyrepeat       Disable keyboard key repeat" << endl;
        cout << "        --nokeypassthrough  Do not send keys directly to textfields" << endl;
#ifdef HAVE_LIRC
        cout << "        --nolirc            Disable LIRC usage" << endl;
#endif
        cout << endl;
        cout << "  If you do not specify xmlfile, eboxy will look for \'" << DEFAULT_XML_FILE << "\' by default." << endl;
        cout << "  See the file README in the eboxy distribution for more information." << endl;
        exit(0);
        break;

      case CLOPT_BPP:
  			d = atoi(optarg);
        break;

      case CLOPT_VERSION:
        cout << "eboxy " << APP_MAJOR_VERSION << "." << APP_MINOR_VERSION << "." << APP_SUB_VERSION << " by Paul Eggleton (bluelightning@bluelightning.org)" << endl;
        cout << "Released under the GNU General Public Licence version 2 or later" << endl;
        exit(0);
        break;

      case CLOPT_NOAUDIO:
        disableaudio = true;
        break;

      case CLOPT_NOKEYPT:
        nokeypassthrough = true;
        break;

      case CLOPT_NOKEYREPEAT:
        nokeyrepeat = true;
        break;

      case CLOPT_SHOWKEYS:
        showkeys = true;
        break;

#ifdef HAVE_LIRC
      case CLOPT_NOLIRC:
        disablelirc = true;
        break;
#endif

      case '?':
        break;

      default:
        abort();

    }
  }

  if(argc - optind > 1) {
    cerr << "You can only specify one XML file" << endl;
    cerr << "Run eboxy --help for more information" << endl;
    exit(1);
  }
  else if((argc - optind) == 1) {
    xmlFile = strdup(argv[optind]);
  }
  else {
    // Check that the default XML file exists
    if(!checkFileExists((char *)DEFAULT_XML_FILE)) {
      cerr << "Run eboxy --help for more information" << endl;
      exit(1);
    }
  }

  // Read key bindings
  if(readKeyConf() != 0) {
    exit(2);
  }

  GUI_Screen *screen = initGUI(w, h, d, v);
  if(screen == NULL)
    return 1;  // something bad happened

  if(xmlFile == NULL)
    xmlFile = strdup((char *)DEFAULT_XML_FILE);

  // Create "system" object
  systemObject = new SystemObject();

  // Initialise mutex for controlling access to processes vector
  pthread_mutex_init(&processmutex, NULL);
  // Initialise mutex for waiting on event processing
  pthread_mutex_init(&eventmutex, NULL);
  // Initialise mutex & condition variable for switching page/file
//X  pthread_mutex_init(&switchmutex, NULL);
//X  pthread_cond_init(&switchcond, NULL);

  if(parseSkin(xmlFile) == 0) {
#ifdef HAVE_LIRC
    if(!disablelirc) {
      // Initialise thread for LIRC events
      if(lirc_init((char *) LIRC_APP_NAME,1)==-1) {
        cerr << "Couldn't initialise LIRC (is lircd running?)" << endl;
        disablelirc = true;
      }
      else {
        if(pthread_create(&lirc_thread, NULL, lirc_function, NULL)) {
          lirc_deinit();
          cerr << "Failed to create LIRC thread" << endl;
          disablelirc = true;
        }
      }
    }
#endif

    systemObject->FireSystemEvent("OnLoad");
    systemObject->InitialisePlugins();
    systemObject->AutoStartTimers();

    // Switch to first page
    switchPage(pages[0]->GetName());

    while(true) {
      int counter = 0;
      garbage = NULL;

      // tell SDL_gui what thread has access to the gui
     	GUI_SetThread(SDL_ThreadID());

      if(hidecursor) {
        SDL_ShowCursor(0);
        SDL_WarpMouse(0,0);
      }
      GUI_SetRunning(1);

      systemObject->FireSystemEvent("OnShow");

      // Process events (mainly interested in keyboard events here)
     	while (GUI_GetRunning()) {
       	SDL_Event event;
     		while (SDL_PollEvent(&event)) {
          bool processed = false;
          if (event.type == SDL_KEYDOWN) {
            if(showkeys) {
              cout << "key scan=" << (int)event.key.keysym.scancode << " sym=" << event.key.keysym.sym;
              cout << " SDLname=\"" << SDL_GetKeyName(event.key.keysym.sym) << "\" eboxyname=\"" << keyToName(event.key.keysym.sym) << "\"" << endl;
            }
            processed = true;
            for(int i=0; i<keybindings.size(); i++) {
              if(event.key.keysym.sym == keybindings[i].key && checkmodkeys(event.key.keysym.mod, keybindings[i].mod)) {
                performAction(keybindings[i].action);
                break;
              }
            }
            if(!nokeypassthrough && event.key.keysym.unicode != 0) {
              SendKeyEvent(event);
            }
          }
          else if (event.type == SDL_USEREVENT) {
            // Handle "posted" events from other threads
            processed = true;
            if(event.user.code == EBSE_SWITCHPAGE) {
              switchPage((const char *)event.user.data1);
//X              pthread_cond_signal(&switchcond);
            }
            else if(event.user.code == EBSE_LOADFILE) {
              switchFile((const char *)event.user.data1, (const char *)event.user.data2);
//X              pthread_cond_signal(&switchcond);
            }
            else if(event.user.code == EBSE_FOCUSNEXT) {
              nextWidget();
            }
            else if(event.user.code == EBSE_FOCUSPREVIOUS) {
              prevWidget();
            }
            else if(event.user.code == EBSE_SELECTFOCUSED) {
              acceptSelection();
            }
            else if(event.user.code == EBSE_LISTNEXT) {
              nextItem();
            }
            else if(event.user.code == EBSE_LISTPREVIOUS) {
              previousItem();
            }
            else if(event.user.code == EBSE_LISTNEXTPAGE) {
              nextItemPage();
            }
            else if(event.user.code == EBSE_LISTPREVIOUSPAGE) {
              previousItemPage();
            }
            else if(event.user.code == EBSE_LISTFIRST) {
              firstItem();
            }
            else if(event.user.code == EBSE_LISTLAST) {
              lastItem();
            }
//            else
//              processed = false;
          }
/*J          else if (event.type == SDL_JOYBUTTONDOWN) {
            printf("debug joystick button down\n");
          }
          else if (event.type == SDL_JOYBUTTONUP) {
            printf("debug joystick button up\n");
          }
          else if (event.type == SDL_JOYAXISMOTION) {
            printf("debug joystick axis\n");
          }
          else if (event.type == SDL_JOYBALLMOTION) {
            printf("debug joystick ball\n");
          }
          else if (event.type == SDL_JOYHATMOTION) {
            printf("debug joystick hat\n");
          }*/

          if(!processed) {
            // Event not handled here, pass it on
    			  GUI_Lock();
            screen->Event(&event, 0, 0);
      			GUI_Unlock();
          }

          if(garbage) {
            // Time to clean up our mess
            garbage->Uninstantiate();
            garbage = NULL;
          }

          if(!hidecursor && currentPage != NULL && event.type == SDL_MOUSEMOTION)
            currentPage->UpdateFocus();

          if(signalexit || signalhide)
            break;
     		}

        // Check if any plugins have asked to be unloaded
        plugin_manager.UnloadRequestingPlugins();

        if(signalexit || signalhide)
          break;

        if(counter > 500) {
          // Housekeeping
          removeFinishedProcesses();
          counter = 0;
        }

     		GUI_Lock();
     		screen->DoUpdate(0);
     		GUI_Unlock();

        counter++;

        // Don't hog the CPU
        SDL_Delay(20);
     	}

      if(signalhide)
        systemObject->FireSystemEvent("OnHide");
      else
        systemObject->FireSystemEvent("OnQuit");

      screen->SetContents(NULL);
      cards->DecRef();
     	screen->DecRef();

      if(hidecursor)
        SDL_ShowCursor(1);

      if(currentPage) {
        lastpagename = strdup(currentPage->GetName()); // need to strdup or we lose it during cleanup
        currentPage = NULL; // required
      }
      else
        lastpagename = NULL;

      if(signalhide)
        unloadObjects(false);
      else
        unloadObjects(true);

      if(garbage) {
        // Time to clean up our mess
        garbage->Uninstantiate();
        delete garbage;
        garbage = NULL;
      }

      GUI_Quit();
      TTF_Quit();
      SDL_Quit();

      if(signalhide) {
        while(signalhide) {
          // Wait
          if(hideprocess != NULL && hideprocess->IsDone()) {
            signalhide = false;
            break;
          }

          // Don't hog the CPU
          SDL_Delay(500);
        }
        if(hideprocess) {
          delete hideprocess;
          hideprocess = NULL;
        }
        // Come back to life
        screen = initGUI(w,h,d,v);
        if(screen == NULL)
          return 1;  // Something bad happened
        switchFile(xmlFile, lastpagename);
        free(lastpagename);
        // Loop back around
      }
      else
        break;
    }

#ifdef HAVE_LIRC
    if(!disablelirc)
      lirc_deinit();
#endif
//J    SDL_JoystickClose(joystick);

    // Clear out key bindings
    while(keybindings.size() > 0) {
      free(keybindings.back().action);
      keybindings.pop_back();
    }
  }
  else
    return 1;

  return 0;
}

#ifdef HAVE_LIRC
void *lirc_function(void *arg) {
  struct lirc_config *config;

  if(lirc_readconfig(NULL, &config, NULL) == 0) {
    char *code;
    char *c;
    int ret;

    while(lirc_nextcode(&code)==0) {
      if(code==NULL)
        continue;
      if(signalhide || ((SDL_GetAppState() & SDL_APPINPUTFOCUS) == 0))
        continue;  // eat events if the app doesn't have input focus
      while((ret = lirc_code2char(config, code, &c))==0 && c != NULL) {
        performAction(c);
      }

      free(code);
      if(ret == -1) break;
	  }
    lirc_freeconfig(config);
  }
	
  pthread_mutex_destroy(&processmutex);
  pthread_mutex_destroy(&eventmutex);

  return 0;
}
#endif

int performAction(const char *action) {
  // We may be in another thread here so we must consider thread-safety
  if(!strcasecmp(action, "prev")) {
    prevWidgetEvent();
  }
  else if(!strcasecmp(action, "next")) {
    nextWidgetEvent();
  }
  else if(!strcasecmp(action, "accept")) {
    acceptSelectionEvent();
  }
  else if(!strcasecmp(action, "quit")) {
    SignalExit();
  }
  else if(!strcasecmp(action, "item_next")) {
    pushUserEvent(EBSE_LISTNEXT, NULL, NULL);
  }
  else if(!strcasecmp(action, "item_prev")) {
    pushUserEvent(EBSE_LISTPREVIOUS, NULL, NULL);
  }
  else if(!strcasecmp(action, "item_nextpage")) {
    pushUserEvent(EBSE_LISTNEXTPAGE, NULL, NULL);
  }
  else if(!strcasecmp(action, "item_prevpage")) {
    pushUserEvent(EBSE_LISTPREVIOUSPAGE, NULL, NULL);
  }
  else if(!strcasecmp(action, "item_first")) {
    pushUserEvent(EBSE_LISTFIRST, NULL, NULL);
  }
  else if(!strcasecmp(action, "item_last")) {
    pushUserEvent(EBSE_LISTLAST, NULL, NULL);
  }
  else if(!strncasecmp(action, "textinput_", 10)) {
    ebWidget *widget = currentPage->GetFocusedWidget();
    if(widget != NULL) {
      if(strcmp(widget->GetType(), TYPESTR_TEXTFIELD) == 0) {
        TextFieldWidget *textfield = (TextFieldWidget *)widget;

        if(isalnum(action[10]) && strlen(action) == 11) {
          // Single alphanumeric character
          textfield->SendChar(action[10]);
          // Reset digit input
          textfield->InputDigit(-1);
        }
        else if(!strcasecmp(action+10, "backspace")) {
          textfield->Backspace();
          // Reset digit input
          textfield->InputDigit(-1);
        }
        else if(!strcasecmp(action+10, "delete")) {
          textfield->DeleteCurrChar();
          // Reset digit input
          textfield->InputDigit(-1);
        }
        else if(!strncasecmp(action+10, "digit", 5) && strlen(action+15) == 1 && isdigit(action[15])) {
          // Digits
          textfield->InputDigit(atoi(action+15));
        }
        else {
          // Symbol
          char c = textInputNameToChar(action + 10);
          if(c != '\0') {
            textfield->SendChar(c);
            // Reset digit input
            textfield->InputDigit(-1);
          }
          else {
            cerr << "Unrecognised action \"" << action << "\"" << endl;
            return 1;
          }
        }
      }
    }
  }
  else {
    if(GetUserAction(action) != NULL) {  // FIXME: this is a bit inefficient
      action_manager.ExecuteAction(action);
    }
    else {
      cerr << "Unrecognised action \"" << action << "\"" << endl;
      return 1;
    }
  }
  return 0;
}

void SignalExit(void) {
  signalexit = true;
}

void SignalHide(void) {
  signalhide = true;
}

ImageEntry *loadImage(const char *filename) {
  return loadImage(filename, false);
}

ImageEntry *loadImage(const char *filename, bool force) {
  char *fullpath;
  ImageEntry *image = NULL;

  if(strlen(filename) == 0)
    return NULL; // hack to avoid errors

  fullpath = expandPath(filename);

  for(int i=0; i<images.size(); i++) {
    if(!strcasecmp(fullpath, images[i]->filename)) {
      // Image already loaded
      if(!force) {
        free(fullpath);
        return images[i];
      }
      else
        image = images[i];
    }
  }

  if(!checkFileExists(fullpath)) {
    free(fullpath);
    throw EBERR_FILE_ERROR;
    return NULL;
  }

  if(force) {
    if(image == NULL) {
      cerr << "Image " << fullpath << " is not already loaded" << endl;
      free(fullpath);
      return NULL;
    }
  }
  else {
    image = new ImageEntry;
  }


  GUI_Surface *imgsurface = new GUI_Surface(fullpath);
  if (imgsurface == NULL) {
    cerr << "Failed to load image " << fullpath << " (file possibly corrupt?)" << endl;
    if(!force)
      delete image;
    free(fullpath);
    return NULL;
  }
  image->surface = imgsurface;

  if(!force) {
    image->filename = fullpath;
    images.push_back(image);
  }
  else
    free(fullpath); // we already have the path set

	return image;
}

FontEntry *loadFont(const char *filename, int size) {
  char *fullpath = NULL;
  FontEntry *font = NULL;

  fullpath = expandPath(filename);

  for(int i=0; i<fonts.size(); i++) {
    if((strcasecmp(fullpath, fonts[i]->filename) == 0) && fonts[i]->size == size) {
      // Font already loaded
      free(fullpath);
      return fonts[i];
    }
  }

  if(!checkFileExists(fullpath)) {
    free(fullpath);
    return NULL;
  }

  font = new FontEntry;

  font->font = new GUI_TrueTypeFont(fullpath, size);
  if (font->font == NULL) {
    cerr << "Failed to load font " << fullpath << " (file possibly corrupt?)" << endl;
    delete font;
    free(fullpath);
    return NULL;
  }
  font->filename = fullpath;
  font->size = size;

  fonts.push_back(font);
	return font;
}

// Check to see if a name is a valid identifier
bool checkName(const char *name) {
  if(name == NULL || strlen(name) == 0)
    return false;

  if(name[0] == '_')  // names may not start with an underscore (reserved)
    return false;

  for(int i=0; i < strlen(name); i++) {
    if(!(isalnum(name[i]) || name[i] == '_'))
      return false;
  }

  // Check for reserved words
  char *chkname = newstring(strlen(name)+3);
  strcpy(chkname, "|");
  strcat(chkname, name);
  strcat(chkname, "|");
  if(strstr("|echo|exec|exechide|execwait|delay|quit|load|loadplugin|page|property|method|event|new|", chkname)) {
    free(chkname);
    return false;
  }
  free(chkname);

  return true;
}

int execute(char *command, bool threaded) {
  Process *process = new Process(command);
  if(threaded) {
    // Save the process object
    pthread_mutex_lock(&processmutex);
    processes.push_back(process);
    pthread_mutex_unlock(&processmutex);
    // Run the process, waiting on it in a separate thread
    process->ExecuteThreaded();
    return 0;
  }
  else {
    // Run the process and wait while it executes
    int retcode = process->Execute();
    delete process;
    return retcode;
  }
}

void executeHide(char *command) {
  SignalHide();
  hideprocess = new Process(command);
  hideprocess->Execute();
}

void removeFinishedProcesses(void) {
  int i=0;
  vector<Process *>::iterator p;

  // Go through the processes vector and remove any that are finished

  pthread_mutex_lock(&processmutex);
  while(i<processes.size()) {
    if(processes[i]->IsDone()) {
      // Process is finished, remove it from the vector
      delete processes[i];
      p = processes.begin() + i;
      processes.erase(p);
    }
    else
      i++;
  }
  pthread_mutex_unlock(&processmutex);
}

// Check if the specified file exists. Prints an error and returns false if it doesn't.
bool checkFileExists(const char *filename) {
  int rc = statfile(filename);

  switch(rc) {
    case ENOENT:
      cerr << "Could not find file '" << filename << "'" << endl;
      return false;
      break;
    case ENOTDIR:
      cerr << "Path in '" << filename << "' does not exist" << endl;
      return false;
      break;
    case EACCES:
      cerr << "Access denied to file '" << filename << "'" << endl;
      return false;
      break;
    case ENAMETOOLONG:
      cerr << "File name '" << filename << "' is too long" << endl;
      return false;
      break;
    default:
      return true;
  }
}

GUI_Screen *initGUI(int width, int height, int depth, int opts) {
	// Initialize SDL
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) < 0) {
    cerr << "Failed to initialise SDL: " << SDL_GetError() << endl;
		return NULL;
  }

  if(!disableaudio) {
    if(SDL_InitSubSystem(SDL_INIT_AUDIO) < 0) {
      cerr << "Failed to initialise SDL audio: " << SDL_GetError() << endl;
      cerr << "(Run with --noaudio if you wish to disable audio usage)" << endl;
    }
  }

/*J  if(SDL_InitSubSystem(SDL_INIT_JOYSTICK) < 0) {
    cerr << "Failed to initialise SDL joystick: " << SDL_GetError() << endl;
  }
  SDL_JoystickEventState(SDL_ENABLE);
  joystick = SDL_JoystickOpen(0);*/

	// Initialize the TTF library
	if (TTF_Init() < 0) {
    cerr << "Failed to initialise TTF font library" << endl;
		return NULL;
  }

	// Initialize the GUI library
	if (GUI_Init() < 0) {
    cerr << "Failed to initialise SDL_gui" << endl;
		return NULL;
  }

  // Create and install the screen (this will attempt to change to the
  // specified video mode)
	GUI_Screen *screen = GUI_ScreenCreate(width, height, depth, opts);
	if (screen == NULL) {
    cerr << "Couldn't initialise SDL video mode " << width << "x" <<  height << "x" << depth << endl;
		return NULL;
  }
 	GUI_SetScreen(screen);

  if(!nokeyrepeat) {
    // Set keyboard repeat
    SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
  }

  // Set window caption
  SDL_WM_SetCaption("eboxy", "eboxy");

  // Initialize the GUI
  cards = new GUI_CardStack("_cardstack", 0, 0, width, height);
  screen->SetContents(cards);

  return screen;
}

ebDynamicObject *findPageObject(const char *name, const char *type) {
  // Find the specified widget/page object
  if(!loadedPage)
    return NULL;

  return loadedPage->FindObject(name, type);
}

bool checkVersion(eboxy_version_info clientlibver, char *filename) {
  // Check a plugin client library version identifier
  if(clientlibver.major > APP_MAJOR_VERSION
      || (clientlibver.major == APP_MAJOR_VERSION && clientlibver.minor > APP_MINOR_VERSION)
      || (clientlibver.major == APP_MAJOR_VERSION && clientlibver.minor == APP_MINOR_VERSION && clientlibver.subminor > APP_SUB_VERSION)) {
    // Too new
    cerr << "Plugin " << filename << " was compiled for a newer version of eboxy." << endl;
    cerr << "Please upgrade eboxy - you will need at least version " << clientlibver.major << "." << clientlibver.minor << "." << clientlibver.subminor << " to use this plugin." << endl;
    return false;
  }
  if(clientlibver.major < PLUGIN_MIN_MAJOR_VERSION
      || (clientlibver.major == PLUGIN_MIN_MAJOR_VERSION && clientlibver.minor < PLUGIN_MIN_MINOR_VERSION)
      || (clientlibver.major == PLUGIN_MIN_MAJOR_VERSION && clientlibver.minor == PLUGIN_MIN_MINOR_VERSION && clientlibver.subminor < PLUGIN_MIN_SUB_VERSION)) {
    // Too old
    cerr << "Plugin " << filename << " was compiled for an older (incompatible) version of eboxy." << endl;
    cerr << "Please obtain an updated version of this plugin." << endl;
    return false;
  }
  return true;
}

const char *getAppVersion(void) {
  return APP_VERSTRING;
}
