/***************************************************************************
                          page.cpp  -  eboxy page (not really a widget)
                             -------------------
    begin                : Sat Aug 3 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>

#include "gui_extensions.h"
#include "utils.h"
#include "timer.h"

#include "page.h"
#include "pluginmanager.h"

using namespace std;

extern PluginManager plugin_manager;
extern const char *TYPESTR_PAGE;
extern const char *TYPESTR_TIMER;
extern const char *TYPESTR_PLUGIN;

ebPage::ebPage(const char *name, int x, int y, int w, int h)
: ebWidget(name, x, y) {
  this->w = w;
  this->h = h;
  this->background = NULL;
  this->type = TYPESTR_PAGE;

  AddProperty("widgetcount", Page_GetWidgetCount, NULL);

  AddMethod("getwidget", 1, Page_GetWidget);
  AddMethod("addwidget", 1, Page_AddWidget);
  AddMethod("removewidget", 1, Page_RemoveWidget);

  AddEvent("OnLoad");
  AddEvent("OnShow");
  AddEvent("OnUnload");
}

GUI_Widget *ebPage::Instantiate(void) {
  // Initialise plugins
  for(int i=0;i<containedobjects.size();i++) {
    ebDynamicObject *object = containedobjects[i];
    if(!strcmp(object->GetType(), TYPESTR_PLUGIN)) {
      // Initialise the plugin
      // FIXME: need to handle errors here!
      plugin_manager.InitialisePlugin((Plugin *)object);
    }
  }

  this->FireEvent("OnLoad");

  GUI_Panel *panel = new GUI_Panel(GetName(), x, y, w, h);  // why does "name" return NULL here, but GetName() works?

  if(background) {
    panel->SetBackground(background->surface);
//  GUI_Widget *bgPicture = new GUI_Picture("gpBackground", x, y, w, h, page->background->surface);
//  panel->AddWidget(bgPicture);
  }

  for(int i=0;i<widgets.size();i++) {
		// create the widget and add it to the page
    ebWidget *widget = widgets[i];
    panel->AddWidget(widget->Instantiate());
  }

  FocusFirstWidget();

  realwidget = panel;

  // We can't fire the OnShow event yet because CurrentPage has not been changed
  // (this is done in OnShow() below, which is called by switchPage)
  return panel;
}

void ebPage::Uninstantiate(void) {
  if(realwidget) {
    this->FireEventWait("OnUnload");
    // Remove widgets from the page
    for(int i=0;i<widgets.size();i++) {
      ebWidget *widget = widgets[i];
      if(widgets[i]->GetInstance())
        realwidget->RemoveWidget(widget->GetInstance());
      widget->Uninstantiate();
    }
    focusedindex = 0;
    // Deal with any owned objects
    for(int i=0;i<containedobjects.size();i++) {
      ebObject *object = containedobjects[i];
      if(!strcmp(object->GetType(), TYPESTR_TIMER)) {
        // Make sure timer is disabled (stopped)
        ((TimerObject *)object)->SetEnabled(false);
      }
      else if(!strcmp(object->GetType(), TYPESTR_PLUGIN)) {
        // Uninitialise the plugin
        plugin_manager.UninitialisePlugin((Plugin *)object);
      }
    }
    ebWidget::Uninstantiate();
  }
}

void ebPage::SetBackground(ImageEntry *image) {
  background = image;
}

void ebPage::FocusNextWidget(void) {
  int check;

  if(widgets.size() == 0) {
    // No widgets
    focusedindex = -1;
    return;
  }

  // Unfocus the currently focused widget, if any
  if(focusedindex >= 0)
    widgets[focusedindex]->LostFocus();

  // Move to next widget
  focusedindex++;
  if(focusedindex > (widgets.size() - 1))
    focusedindex = 0;

  // Now we need to make sure the next widget is focusable, if not find the next focusable one
  check = focusedindex;
  while(focusedindex > -1 && !((widgets[focusedindex]->GetTabStop()) && widgets[focusedindex]->IsEnabled())) {
    focusedindex++;
    if(focusedindex > (widgets.size() - 1))
      focusedindex = 0;
    if(focusedindex == check)  // no more left, can't set focus
      focusedindex = -1;
  }

  // Finally, give focus to the newly focused widget, if any
  if(focusedindex >= 0)
    widgets[focusedindex]->GotFocus();
}

void ebPage::FocusPreviousWidget(void) {
  int check;

  if(widgets.size() == 0) {
    // No widgets
    focusedindex = -1;
    return;
  }

  // Unfocus the currently focused widget, if any
  if(focusedindex >= 0)
    widgets[focusedindex]->LostFocus();

  // Move to previous widget
  focusedindex--;
  if(focusedindex < 0)
    focusedindex = widgets.size() - 1;

  // Now we need to make sure the previous widget is focusable, if not find the previous focusable one
  check = focusedindex;
  while(focusedindex > -1 && !((widgets[focusedindex]->GetTabStop()) && widgets[focusedindex]->IsEnabled())) {
    focusedindex--;
    if(focusedindex < 0)
      focusedindex = widgets.size() - 1;
    if(focusedindex == check)  // no more left, can't set focus
      focusedindex = -1;
  }

  // Finally, give focus to the newly focused widget, if any
  if(focusedindex >= 0)
    widgets[focusedindex]->GotFocus();
}

void ebPage::UpdateFocus(void) {
  int newfocus = -1;

  if(widgets.size() == 0) {
    // No widgets
    focusedindex = -1;
    return;
  }

  if(realwidget->GetFlags() & WIDGET_PRESSED)  // makes sure we don't get silly stuff happening when the mouse is down
    return;

  // Make sure there is a focused widget
  if(focusedindex < 0 || !((widgets[focusedindex]->GetTabStop()) && widgets[focusedindex]->IsEnabled()))
    FocusNextWidget();
  else {
    for(int i=0; i<widgets.size(); i++) {
      if(widgets[i]->IsGUIFocused() && ((widgets[i]->GetTabStop()) && widgets[i]->IsEnabled()) ) {
        newfocus = i;
        break;
      }
    }
    if(newfocus < 0) {
      // SDL_gui - mouse has taken away focus
      widgets[focusedindex]->GotFocus();
    }
    else if(focusedindex != newfocus) {
      widgets[focusedindex]->LostFocus();
      focusedindex = newfocus;
      widgets[focusedindex]->GotFocus();
    }
  }
}

void ebPage::FocusFirstWidget(void) {
  // Give focus to first widget
  focusedindex = -1;
  FocusNextWidget();
}

void ebPage::SetFocus(ebWidget *widget) {
  // Find widget in array
  for(int i=0; i<widgets.size(); i++) {
    if(widgets[i]==widget) {
      if(i != focusedindex && widgets[i]->GetTabStop() && widgets[i]->IsEnabled() ) {
        // Give widget focus
        widgets[focusedindex]->LostFocus();
        focusedindex = i;
        widgets[focusedindex]->GotFocus();
      }
      break;
    }
  }
}

void ebPage::AddWidget(ebWidget *widget) {
  for(int i=0;i<widgets.size();i++) {
    if(widgets[i] == widget) {
      cerr << "AddWidget: widget " << widget->GetName() << " is already added to the page " << GetName() << endl;
      return;
    }
  }

  // Add widget to widgets array
  widgets.push_back(widget);
  widget->SetParent(this);

  if(realwidget) {
    // Adding at runtime
    GUI_Widget *rwidget = widget->Instantiate();
    if(rwidget)
      ((GUI_Panel *)realwidget)->AddWidget(rwidget);
    else
      cerr << "AddWidget: widget " << widget->GetName() << " could not be instantiated" << endl;
  }
}

void ebPage::RemoveWidget(ebWidget *widget) {
  for(int i=0;i<widgets.size();i++) {
    if(widgets[i] == widget) {
      if(realwidget) {
        // Removing at runtime
        ((GUI_Panel *)realwidget)->RemoveWidget(widget->GetInstance());
        widget->Uninstantiate();
      }

      vector<ebWidget *>::iterator p = widgets.begin() + i;
      widgets.erase(p);

      realwidget->MarkChanged();
      break;
    }
  }
}

ebWidget *ebPage::GetFocusedWidget(void) {
  if(focusedindex >= 0)
    return widgets[focusedindex];
  else
    return NULL;
}

ebDynamicObject *ebPage::FindObject(const char *name, const char *type) {
  ebWidget *widget = FindWidget(name, type);
  if(widget)
    return widget;
  else
    return ObjectContainer::FindObject(name, type);
}

ebWidget *ebPage::FindWidget(const char *name, const char *type) {
  for(int i=0; i<widgets.size(); i++) {
    if(!strcasecmp(widgets[i]->GetName(), name) && (type == NULL || (!strcasecmp(widgets[i]->GetType(), type))))
      return widgets[i];
  }
  return NULL;
}

void ebPage::InvalidateRect(SDL_Rect a) {
  // Mark all the widgets in the specified area as changed so that they will be redrawn
  // FIXME: This is not much use right now, because it can get into an endless loop and
  //   stuff isn't done in the correct order. It requires substantial changes to the way
  //   SDL_gui works in order to fix it, though.
  if(realwidget) {
    for(int i=0; i<widgets.size(); i++) {
      if(rectsintersect(a, widgets[i]->GetArea())) {
        // Widget is inside specified area, invalidate it
        widgets[i]->Invalidate();
      }
    }
  }
}

void ebPage::OnShow(void) {
  // Called by switchPage when the page has done switching
  this->FireEvent("OnShow");

  // Start any auto-starting timers
  for(int i=0;i<containedobjects.size();i++) {
    ebDynamicObject *object = containedobjects[i];
    if(!strcmp(object->GetType(), TYPESTR_TIMER)) {
      // Start the timer
      ((TimerObject *)object)->AutoStart();
    }
  }
}

/* Property get/set functions */

//-- widgetcount

const char *Page_GetWidgetCount(ebCustomObject *sender) {
  return inttostr( ((ebPage *)sender)->widgets.size() );
}

/* Method functions */

const char *Page_AddWidget(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  ebPage *page = (ebPage *)sender;

  ebWidget *widget = plugin_manager.FindTempWidget(args[0]);

  if(widget)
    page->AddWidget(widget);
  else
    cerr << "Page.AddWidget: Unable to find widget named " << args[0] << " in constructed widgets" << endl;
  
  return "";
}

const char *Page_RemoveWidget(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  ebPage *page = (ebPage *)sender;

  ebWidget *widget = page->FindWidget(args[0], NULL);
  if(widget) {
    if(plugin_manager.FindTempWidget(args[0]) != widget)
      cerr << "Page.RemoveWidget: widget " << args[0] << " cannot be removed - was not constructed" << endl;
    else
      page->RemoveWidget(widget);
  }
  else
    cerr << "Page.RemoveWidget: no widget named " << args[0] << " exists on page " << page->GetName() << endl;
 
  return "";
}

const char *Page_GetWidget(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  ebPage *page = (ebPage *)sender;

  int index = atoi(args[0]);
  if(index >= 0 && index < page->widgets.size())
    return page->widgets[index]->GetName();
  else
    cerr << "Page.GetWidget: index " << index << " is out of bounds" << endl;

  return "";
}
