/***************************************************************************
                          picturewidget.cpp  -  eboxy static image widget
                             -------------------
    begin                : Sat Aug 3 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <cstring>
#include <cstdlib>
#include "SDL_gui.h"

#include "widgets.h"
#include "main.h"

#include "picturewidget.h"

using namespace std;
extern const char *TYPESTR_PICTURE;

PictureWidget::PictureWidget(const char *name, int x, int y, ImageEntry *image)
: ebWidget(name, x, y) {
  this->image = image;
  this->type = TYPESTR_PICTURE;

  AddProperty("image", PictureWidget_GetImage, PictureWidget_SetImage, true);

  AddMethod("reloadimage", 0, PictureWidget_ReloadImage);
}

GUI_Widget *PictureWidget::Instantiate(void) {
  if(image) {
    GUI_Picture *pic = new GUI_Picture(name, x, y, w, h, image->surface);
    realwidget = pic;
    realwidget->MarkChanged(); // for some strange reason this is required if you set image after construction
    return pic;
  }
  else {
    cerr << "PictureWidget::Instantiate - image property must be set" << endl;
    return NULL;
  }
}

void PictureWidget::SetImage(const char *filename) {
  ImageEntry *image = loadImage(filename);
  if(image) {
    this->image = image;
    SetWidth(image->surface->GetWidth());
    SetHeight(image->surface->GetHeight());
    if(realwidget) {
      ((GUI_Picture *)realwidget)->SetImage(image->surface);
      realwidget->MarkChanged();
    }
  }
}

void PictureWidget::ReloadImage(void) {
  if(this->image) {
    ImageEntry *image = loadImage(this->image->filename, true);
    if(image) {
      this->image = image;
      if(realwidget) {
        ((GUI_Picture *)realwidget)->SetImage(image->surface);
        realwidget->MarkChanged();
      }
    }
  }
}

// Property set/get functions

// -- image

const char *PictureWidget_GetImage(ebCustomObject *sender) {
  if( ((PictureWidget *)sender)->image )
    return( ((PictureWidget *)sender)->image->filename );
  else
    return "";
}

void PictureWidget_SetImage(ebCustomObject *sender, const char *value) {
  ((PictureWidget *)sender)->SetImage(value);
}

// Method functions

const char *PictureWidget_ReloadImage(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  ((PictureWidget *)sender)->ReloadImage();
  return "";
}

