/***************************************************************************
                          pluginmanager.h  -  eboxy plugin manager header
                             -------------------
    begin                : Thu Sep 5 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INCLUDED__PLUGINMANAGER_H
#define INCLUDED__PLUGINMANAGER_H

#include "widgets.h"
#include "plugintypes.h"
#include "pluginobject.h"
#include "customobject.h"
#include "page.h"

extern "C" {
void *resolveFunction(const char *name);
int findObject(const char *object, const char *type);
const char *typeOfObject(const char *object);
const char *getPropertyAsString(const char *object, const char *property);
void setPropertyAsString(const char *object, const char *property, const char *value);
const char *callMethod(const char *object, const char *method, int numargs, const char *args[]);
int registerObject(const char *name, int plugin_id);
int registerPropertyDL(const char *object, const char *property, int plugin_id, const char *getter, const char *setter);
int registerEventHandlerDL(const char *object, const char *event, int plugin_id, const char *eventfunc);
int registerMethodDL(const char *object, const char *method, int plugin_id, int numargs, const char *methodfunc);
int fireEvent(const char *object, const char *event, int plugin_id);
const plugin_info *getPluginInfo(const char *name);
void setPluginInfo(int plugin_id, const char *namestring, const char *verstring);
void requestUnload(int plugin_id);
int changePage(const char *pagename);
int loadXMLFile(const char *filename, const char *pagename);
int performActionAPI(const char *actionname);
int unregisterObject(const char *name, int plugin_id);
int unregisterEventHandlerDL(const char *object, const char *event, int plugin_id);
int createWidget(const char *name, const char *type, const char *templatename);
int createPage(const char *name);
int cloneWidget(const char *original, const char *duplicate);
int deleteWidget(const char *name);
int deletePage(const char *name);
int runScript(const char *code, int flags);
}

class PluginManager {
  public:
    PluginManager(void);
    virtual ~PluginManager(void);
    int LoadPlugin(const char *name, bool initialise);
    int LoadPlugin(const char *filename, bool initialise, Plugin **pluginref);
    int InitialisePlugin(Plugin *plugin);
    void UninitialisePlugin(Plugin *plugin);
    void UnloadAllPlugins(void);
    void UnloadAllCustomObjects(void);
    ebDynamicObject *FindObject(const char *name);
    ebCustomObject *FindCustomObject(const char *name);
    ebWidget *FindTempWidget(const char *name);
    int FindDynamicPage(ebPage *page);
    ebPage *FindPage(const char *name);
    Plugin *FindPlugin(int plugin_id);
    Plugin *FindPlugin(const char *name);
    ebDynamicObject *FindExtPageObject(const char *name);
    void UnloadRequestingPlugins(void);
    void BroadcastMessage(int msgcode, void *msgdata1, void *msgdata2);
    int RegisterObject(const char *name, Plugin *plugin);
    int UnregisterObject(const char *name, Plugin *plugin);
    void RequestUnload(int plugin_id);
    void PrintPluginList(void);
  private:
    vector<Plugin *> plugins;
    vector<ebCustomObject *> objects;  // store for pseudo-objects created by plugins
    bool requestedunload;
};

#endif
