/***************************************************************************
                  pluginobject.h - eboxy-side plugin wrapper object
                             -------------------
    begin                : Thu May 22 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INCLUDED__PLUGINOBJECT_H
#define INCLUDED__PLUGINOBJECT_H

#include "types.h"
#include "dynamicobject.h"
#include "plugintypes.h"
#include "methodhandler.h"
  
using namespace std;

class Plugin: public ebDynamicObject {
  public:
    Plugin(const char *name, int id, void *library);
    virtual ~Plugin(void);
    void BeforeInitialise(void);
    void AfterInitialise(void);
    int Initialise(void);
    int Uninitialise(void);
    int Message(int msgcode, void *msgdata1, void *msgdata2);
    int GetState(void);
    int GetID(void);
    const plugin_info *GetInfo(void);
    void SetInfo(const char *namestring, const char *verstring);
    void *GetFunction(const char *funcname);
    void RequestUnload(void);
    bool IsRequestingUnload(void);
    void AddEventHandler(EventTemplate *evt);
    void RemoveEventHandler(EventTemplate *evt);
    void ObjectRegistered(ebCustomObject *obj); // called by plugin manager when an object is registered by the plugin
    // The following methods handle handlers owned by this plugin (on other objects)
    void AddOwnedEventHandler(EventHandler *handler);
    void AddOwnedMethodHandler(MethodHandler *handler);
    void RemoveOwnedEventHandler(EventHandler *handler);
    void RemoveOwnedMethodHandler(MethodHandler *handler);
  private:
    int plugin_id;
    int (*ebplugin_message)(int, void *, void *);
    void (*ebplugin_deinit)(void);
    void (*clientlib_deinit)(void);
    void *library;
    plugin_info info;
    int state;
    bool requestedunload;
    vector<EventTemplate *> myeventhandlers;  // handlers for events generated by the plugin - these are persistent
    vector<EventHandler *> ownedeventhandlers; // handlers for events generated by other objects which the plugin is interested in
    vector<MethodHandler *> ownedmethodhandlers;
    pthread_mutex_t init_mutex;
};

#endif
