/***************************************************************************
               ebscript_flex.l  -  flex input for eboxy script parser
                             -------------------                                         
    begin                : Sat Sep 7 2002                                           
    copyright            : (C) 2002 by Paul Eggleton                         
    email                : bluelightning@bluelightning.org                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

%option noyywrap
%option yylineno
%option c++
%option yyclass="ScriptLexer"

%{
#include <iostream>
#include <strstream.h>  // Deprecated, but required for flex string parsing
#include <cstring>

#include "script.h"

char string_buf[MAX_STR_CONST];
char *string_buf_ptr;
YY_BUFFER_STATE last_buf_state;

%}

ID      [a-z_][a-z0-9_]*
DIGIT   [0-9]

%x str

%%
\"      {
          string_buf_ptr = string_buf;
          BEGIN(str);
        }

<str>\" { /* closing quote */
          BEGIN(INITIAL);
          *string_buf_ptr = '\0';
          semantic_value.string = strdup(string_buf); /* Don't forget to free! */
          return tSTRING;
        }
<str>\n {
          cerr << "Script error: Unterminated string constant at line " << yylineno << endl;
          BEGIN(INITIAL);
        }
<str>\\n  *string_buf_ptr++ = '\n';
<str>\\t  *string_buf_ptr++ = '\t';
<str>\\r  *string_buf_ptr++ = '\r';
<str>\\b  *string_buf_ptr++ = '\b';
<str>\\f  *string_buf_ptr++ = '\f';
<str>\\(.|\n)  *string_buf_ptr++ = yytext[1];
<str>[^\\\n\"]+ {
          char *yptr = yytext;
          while ( *yptr )
            *string_buf_ptr++ = *yptr++;
        }

{DIGIT}+    return tNUMBER;
echo        return tECHO;
exechide    return tEXECHIDE;
execwait    return tEXECWAIT;
exec        return tEXEC;
load        return tLOAD;
loadplugin  return tLOADPLUGIN;
page        return tPAGE;
quit        return tQUIT;
delay       return tDELAY;
=           return tASSIGN;
\+          return tPLUS;
\.          return tDOT;
\(          return tLBRACKET;
\)          return tRBRACKET;
\,          return tCOMMA;
{ID}        {
              semantic_value.string = strdup(yytext);  /* Don't forget to free! */
              return tIDENT;
            }
#[^\n]*     return tCOMMENT; /* this allows us to have completely commented out scripts */
[ \t\n]+    /* eat whitespace */
%%

void ScriptLexer::embedded_yylex(int& result, YYSTYPE *lvalp){
  result = yylex();
  *lvalp = semantic_value;
}

ScriptLexer::ScriptLexer(char *string, int flags) {
  // Initialise variables
  this->lineoffset = 0;
  this->filename = NULL;
  this->flags = flags;
  // Load string stream
  mem_buff = new istrstream(string, strlen(string));
  mem_input = new istream(mem_buff->rdbuf());
  switch_streams(mem_input, NULL);
}

ScriptLexer::~ScriptLexer(void) {
  if(this->filename)
    free(this->filename);
  delete mem_buff;
  delete mem_input;
}

void ScriptLexer::SetLineOffset(int line) {
  this->lineoffset = line;
}

int ScriptLexer::GetLineNumber(void) {
  return lineno() + lineoffset - 1;
}

void ScriptLexer::SetFileName(const char *filename) {
  if(this->filename)
    free(this->filename);
  this->filename = strdup(filename);
}

const char *ScriptLexer::GetFileName(void) {
  return filename;
}

bool ScriptLexer::IsTrusted(void) {
  return flags & SCRIPTFLAGS_TRUSTED == SCRIPTFLAGS_TRUSTED;
}
