/***************************************************************************
                          systemobject.cpp  -  global system object
                             -------------------
    begin                : Thu Sep 26 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <cstring>
#include <cstdlib>

#include "SDL_gui.h"
#include "utils.h"
#include "timer.h"
#include "main.h"

#include "systemobject.h"
#include "pluginmanager.h"

using namespace std;

extern char *xmlFile;
extern PluginManager plugin_manager;
extern vector<ebPage *> pages;
extern const char *TYPESTR_TIMER;
extern const char *TYPESTR_PLUGIN;

SystemObject::SystemObject(void)
: ebCustomObject("system") {
  this->type = "SystemObject";

  AddProperty("screenwidth", SystemObject_GetScreenWidth, NULL);
  AddProperty("screenheight", SystemObject_GetScreenHeight, NULL);
  AddProperty("currentfile", SystemObject_GetCurrentFile, NULL);
  AddProperty("version", SystemObject_GetVersion, NULL);
  AddProperty("pagecount", SystemObject_GetPageCount, NULL);

  AddMethod("quit", 0, SystemObject_Quit);
  AddMethod("focusnext", 0, SystemObject_FocusNext);
  AddMethod("focusprevious", 0, SystemObject_FocusPrevious);
  AddMethod("selectfocused", 0, SystemObject_SelectFocused);
  AddMethod("printpluginlist", 0, SystemObject_PrintPluginList);
  AddMethod("performaction", 1, SystemObject_PerformAction);
  AddMethod("getpage", 1, SystemObject_GetPage);

  AddEvent("OnLoad");
  AddEvent("OnShow");
  AddEvent("OnHide");
  AddEvent("OnUnload");
}

SystemObject::~SystemObject(void) {
  ClearEventHandlers();
  ClearObjects();
}

int SystemObject::FireSystemEvent(const char *event) {
  // All system events are non-threaded
  return FireEventWait(event);
}

void SystemObject::AutoStartTimers(void) {
  // Start any auto-starting timers
  for(int i=0;i<containedobjects.size();i++) {
    ebDynamicObject *object = containedobjects[i];
    if(!strcmp(object->GetType(), TYPESTR_TIMER)) {
      // Start the timer
      ((TimerObject *)object)->AutoStart();
    }
  }
}

void SystemObject::InitialisePlugins(void) {
  // Initialise plugins
  for(int i=0;i<containedobjects.size();i++) {
    ebDynamicObject *object = containedobjects[i];
    if(!strcmp(object->GetType(), TYPESTR_PLUGIN)) {
      // Initialise the plugin
      plugin_manager.InitialisePlugin((Plugin *)object);
    }
  }
}

/* Property get/set functions */

// -- screenwidth --

const char *SystemObject_GetScreenWidth(ebCustomObject *sender) {
  return inttostr(GUI_GetScreen()->GetWidth());
}

// -- screenheight --

const char *SystemObject_GetScreenHeight(ebCustomObject *sender) {
  return inttostr(GUI_GetScreen()->GetHeight());
}

// -- currentfile --

const char *SystemObject_GetCurrentFile(ebCustomObject *sender) {
  return xmlFile;
}

// -- version --

const char *SystemObject_GetVersion(ebCustomObject *sender) {
  return getAppVersion();
}

// -- pagecount --

const char *SystemObject_GetPageCount(ebCustomObject *sender) {
  return inttostr(pages.size());
}

/* Method functions */

const char *SystemObject_Quit(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  SignalExit();
  return "";
}

const char *SystemObject_FocusNext(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  nextWidgetEvent();
  return "";
}

const char *SystemObject_FocusPrevious(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  prevWidgetEvent();
  return "";
}

const char *SystemObject_SelectFocused(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  acceptSelectionEvent();
  return "";
}

const char *SystemObject_PrintPluginList(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  plugin_manager.PrintPluginList();
  return "";
}

const char *SystemObject_PerformAction(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  performAction(args[0]);
  return "";
}

const char *SystemObject_GetPage(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  int index = atoi(args[0]);
  if(index >= 0 && index < pages.size())
    return pages[index]->GetName();
  else {
    cerr << "system.getpage: index " << index << " is out of bounds" << endl;
    return "";
  }
}

