/***************************************************************************
                       togglebuttonwidget.cpp  -  toggle button widget
                             -------------------
    begin                : Thu Feb 6 2003
    copyright            : (C) 2003 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>

#include "actionmanager.h"
#include "errors.h"
#include "utils.h"
#include "main.h"

#include "togglebuttonwidget.h"
#include "page.h"

using namespace std;

// FIXME: this code is largely duplicated from ButtonWidget. This needs to be refactored.

extern ebPage *currentPage;
extern ActionManager action_manager;
extern const char *TYPESTR_TOGGLEBUTTON;

ToggleButtonWidget::ToggleButtonWidget(const char *name, int x, int y, int w, int h)
: ebWidget(name, x, y) {
  this->w = w;
  this->h = h;
  // Set defaults
  tabstop = true;
  action = NULL;
  normalimage = NULL;
  normalfocusimage = NULL;
  checkedimage = NULL;
  checkedfocusimage = NULL;
//X  disabledimage = NULL;
  enabled = true;
  checked = false;
  this->type = TYPESTR_TOGGLEBUTTON;

  AddProperty("normalimage", ToggleButtonWidget_GetNormalImage, ToggleButtonWidget_SetNormalImage);
  AddProperty("normalfocusimage", ToggleButtonWidget_GetNormalFocusImage, ToggleButtonWidget_SetNormalFocusImage);
  AddProperty("checkedimage", ToggleButtonWidget_GetCheckedImage, ToggleButtonWidget_SetCheckedImage);
  AddProperty("checkedfocusimage", ToggleButtonWidget_GetCheckedFocusImage, ToggleButtonWidget_SetCheckedFocusImage);
  AddProperty("enabled", ToggleButtonWidget_GetEnabled, ToggleButtonWidget_SetEnabled);
  AddProperty("checked", ToggleButtonWidget_GetChecked, ToggleButtonWidget_SetChecked);

  AddEvent("OnClick");
  AddEvent("OnGotFocus");
  AddEvent("OnLostFocus");
}

ToggleButtonWidget::~ToggleButtonWidget(void) {
  //
}

GUI_Widget *ToggleButtonWidget::Instantiate(void) {
  GUI_ToggleButton *button = new GUI_ToggleButton(name, x, y, w, h);

  if(normalimage)
    button->SetOffNormalImage(normalimage->surface);
  if(normalfocusimage)
    button->SetOffHighlightImage(normalfocusimage->surface);
  if(checkedimage)
    button->SetOnNormalImage(checkedimage->surface);
  if(checkedfocusimage)
    button->SetOnHighlightImage(checkedfocusimage->surface);

  if(!enabled)
    button->SetFlags(WIDGET_DISABLED);
  else
    button->ClearFlags(WIDGET_DISABLED);

  if(checked)
    button->SetFlags(WIDGET_TURNED_ON);

  GUI_EventHandler<ToggleButtonWidget> *cb = new GUI_EventHandler<ToggleButtonWidget>(this, &ToggleButtonWidget::OnClick);
  button->SetClick(cb);
	cb->DecRef();

  if(action) {
    int rc = action_manager.RegisterLocalAction(action, this);
    if(rc == EBERR_ACTION_INVALID) {
      cerr << "Warning: action \"" << action << "\" is not a valid user action" << endl;
    }
    else if(rc == EBERR_ACTION_ASSIGNED) {
      cerr << "Warning: action \"" << action << "\" is assigned to more than one widget in this context" << endl;
    }
  }

  realwidget = button;
  return button;
}

void ToggleButtonWidget::Uninstantiate(void) {
  if(realwidget) {
    focused = false;

    if(action)
      action_manager.UnregisterLocalAction(action, this);
  }
  ebWidget::Uninstantiate();
}

void ToggleButtonWidget::SetWidth(int value) {
  if(value == -1 && normalimage != NULL)
    value = normalimage->surface->GetWidth();
  ebWidget::SetWidth(value);
}

void ToggleButtonWidget::SetHeight(int value) {
  if(value == -1 && normalimage != NULL)
    value = normalimage->surface->GetHeight();
  ebWidget::SetHeight(value);
}

void ToggleButtonWidget::SetNormalImage(ImageEntry *image) {
  if(image) {
    this->normalimage = image;
    if(realwidget)
      ((GUI_ToggleButton *)realwidget)->SetOffNormalImage(image->surface);
    if(this->w=-1)
      SetWidth(-1);
    if(this->h=-1)
      SetHeight(-1);
  }
}

void ToggleButtonWidget::SetNormalFocusImage(ImageEntry *image) {
  if(image) {
    this->normalfocusimage = image;
    if(realwidget)
      ((GUI_ToggleButton *)realwidget)->SetOffHighlightImage(image->surface);
  }
}

void ToggleButtonWidget::SetCheckedImage(ImageEntry *image) {
  if(image) {
    this->checkedimage = image;
    if(realwidget)
      ((GUI_ToggleButton *)realwidget)->SetOnNormalImage(image->surface);
  }
}

void ToggleButtonWidget::SetCheckedFocusImage(ImageEntry *image) {
  if(image) {
    this->checkedfocusimage = image;
    if(realwidget)
      ((GUI_ToggleButton *)realwidget)->SetOnHighlightImage(image->surface);
  }
}

void ToggleButtonWidget::SetAction(const char *actionname) {
  this->action = GetUserAction(actionname);
}

void ToggleButtonWidget::SetEnabled(bool enabled) {
  this->enabled = enabled;
  if(realwidget) {
    if(!enabled) {
      ((GUI_ToggleButton *)realwidget)->SetFlags(WIDGET_DISABLED);
      if(focused && currentPage != NULL) {
        // If the widget is disabled and has the focus, we want to give the focus to some other widget
        currentPage->UpdateFocus();
      }
    }
    else
      ((GUI_ToggleButton *)realwidget)->ClearFlags(WIDGET_DISABLED);
  }
}

bool ToggleButtonWidget::GetChecked(void) {
  return checked;
}

void ToggleButtonWidget::SetChecked(bool checked) {
  this->checked = checked;
  if(realwidget) {
    if(checked)
      realwidget->SetFlags(WIDGET_TURNED_ON);
    else
      realwidget->ClearFlags(WIDGET_TURNED_ON);
  }
}

void ToggleButtonWidget::OnClick(GUI_Object *sender) {
  if(realwidget->GetFlags() & WIDGET_TURNED_ON)
    this->checked = true;
  else
    this->checked = false;

  this->FireEvent("OnClick");
}

void ToggleButtonWidget::GotFocus(void) {
  if(!focused) {
    this->FireEvent("OnGotFocus");
    if(normalfocusimage != NULL && checkedfocusimage != NULL) {
      ((GUI_ToggleButton *)realwidget)->SetOnNormalImage(checkedfocusimage->surface);
      ((GUI_ToggleButton *)realwidget)->SetOffNormalImage(normalfocusimage->surface);
    }
    ebWidget::GotFocus();
  }
}

void ToggleButtonWidget::LostFocus(void) {
  if(focused) {
    this->FireEvent("OnLostFocus");
    if(normalimage != NULL && checkedimage != NULL) {
      ((GUI_ToggleButton *)realwidget)->SetOnNormalImage(checkedimage->surface);
      ((GUI_ToggleButton *)realwidget)->SetOffNormalImage(normalimage->surface);
    }
    ebWidget::LostFocus();
  }
}

void ToggleButtonWidget::Execute(void) {
  SetChecked(!GetChecked());
}

// Properties

// -- normalimage

const char *ToggleButtonWidget_GetNormalImage(ebCustomObject *sender) {
  if( ((ToggleButtonWidget *)sender)->normalimage )
    return ((ToggleButtonWidget *)sender)->normalimage->filename;
  else
    return "";
}

void ToggleButtonWidget_SetNormalImage(ebCustomObject *sender, const char *value) {
  ImageEntry *image = loadImage(value);
  if(image)
    ((ToggleButtonWidget *)sender)->SetNormalImage(image);
}

// -- normalfocusimage

const char *ToggleButtonWidget_GetNormalFocusImage(ebCustomObject *sender) {
  if( ((ToggleButtonWidget *)sender)->normalfocusimage )
    return ((ToggleButtonWidget *)sender)->normalfocusimage->filename;
  else
    return "";
}

void ToggleButtonWidget_SetNormalFocusImage(ebCustomObject *sender, const char *value) {
  ImageEntry *image = loadImage(value);
  if(image)
    ((ToggleButtonWidget *)sender)->SetNormalFocusImage(image);
}

// -- checkedimage

const char *ToggleButtonWidget_GetCheckedImage(ebCustomObject *sender) {
  if( ((ToggleButtonWidget *)sender)->checkedimage )
    return ((ToggleButtonWidget *)sender)->checkedimage->filename;
  else
    return "";
}

void ToggleButtonWidget_SetCheckedImage(ebCustomObject *sender, const char *value) {
  ImageEntry *image = loadImage(value);
  if(image)
    ((ToggleButtonWidget *)sender)->SetCheckedImage(image);
}

// -- checkedfocusimage

const char *ToggleButtonWidget_GetCheckedFocusImage(ebCustomObject *sender) {
  if( ((ToggleButtonWidget *)sender)->checkedfocusimage )
    return ((ToggleButtonWidget *)sender)->checkedfocusimage->filename;
  else
    return "";
}

void ToggleButtonWidget_SetCheckedFocusImage(ebCustomObject *sender, const char *value) {
  ImageEntry *image = loadImage(value);
  if(image)
    ((ToggleButtonWidget *)sender)->SetCheckedFocusImage(image);
}

// -- enabled --

const char *ToggleButtonWidget_GetEnabled(ebCustomObject *sender) {
  return(booltostr( ((ToggleButtonWidget *)sender)->enabled ));
}

void ToggleButtonWidget_SetEnabled(ebCustomObject *sender, const char *value) {
  ((ToggleButtonWidget *)sender)->SetEnabled(strtobool(value));
}

// -- checked --

const char *ToggleButtonWidget_GetChecked(ebCustomObject *sender) {
  return(booltostr( ((ToggleButtonWidget *)sender)->GetChecked() ));
}

void ToggleButtonWidget_SetChecked(ebCustomObject *sender, const char *value) {
  ((ToggleButtonWidget *)sender)->SetChecked(strtobool(value));
}
