/***************************************************************************
                    widget.cpp  -  eboxy widget parent class
                             -------------------
    begin                : Sat Aug 3 2002
    copyright            : (C) 2002 by Paul Eggleton
    email                : bluelightning@bluelightning.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <cstring>
#include <cstdlib>
#include <iostream>
#include "SDL_gui.h"

#include "widgets.h"
#include "page.h"
#include "utils.h"

using namespace std;

extern ebPage *currentPage;

ebWidget::ebWidget(const char *name, int x, int y)
: ebCustomObject(name) {
  this->x = x;
  this->y = y;
  this->tabstop = false; // default
  realwidget = NULL;
  focused = false;

  AddProperty("x", Widget_GetX, Widget_SetX);
  AddProperty("y", Widget_GetY, Widget_SetY);
  AddProperty("width", Widget_GetWidth, Widget_SetWidth);
  AddProperty("height", Widget_GetHeight, Widget_SetHeight);

  AddMethod("setfocus", 0, Widget_SetFocus);
}

ebWidget::~ebWidget(void) {
  if(realwidget)
    Uninstantiate();
}

void ebWidget::SetParent(ebWidget *p) {
  this->parent = p;
}

bool ebWidget::GetTabStop(void) {
  return tabstop;
}

void ebWidget::SetTabStop(bool value) {
  tabstop = value;
}

void ebWidget::GotFocus(void) {
  if(realwidget) {
    realwidget->SetFlags(WIDGET_INSIDE);
    GUI_GetScreen()->SetFocusWidget(realwidget);
  }
  focused = true;
}

void ebWidget::LostFocus(void) {
  if(realwidget)
    realwidget->ClearFlags(WIDGET_INSIDE);
  focused = false;
}

bool ebWidget::IsGUIFocused(void) {
  if(realwidget) {
    if(realwidget->GetFlags() & WIDGET_INSIDE)
      return true;
    else
      return false;
  }
  return false;
}

bool ebWidget::IsEnabled(void) {
  if(realwidget) {
    if(!(realwidget->GetFlags() & WIDGET_DISABLED))
      return true;
  }
  return false;
}

void ebWidget::SetX(int value) {
  SDL_Rect a;
  a = GetArea();
  this->SetPosition(value, a.y);
}

void ebWidget::SetY(int value) {
  SDL_Rect a;
  a = GetArea();
  this->SetPosition(a.x, value);
}

void ebWidget::SetPosition(int x, int y) {
  this->x = x;
  this->y = y;
  if(realwidget) {
    SDL_Rect a;
    a = GetArea();
    GUI_Lock();
    realwidget->GetParent()->Erase(&a);
    ((ebPage *)parent)->InvalidateRect(a);
    realwidget->SetPosition(x, y);
    this->Invalidate();
    GUI_Unlock();
  }
}

void ebWidget::SetWidth(int value) {
  this->w = value;
}

void ebWidget::SetHeight(int value) {
  this->h = value;
}

void ebWidget::Uninstantiate(void) {
  if(realwidget) {
    realwidget->DecRef();
    realwidget = NULL;
  }
}

GUI_Widget *ebWidget::GetInstance(void) {
  return realwidget;
}

SDL_Rect ebWidget::GetArea(void) {
  if(realwidget)
    return realwidget->GetArea();
  else {
    SDL_Rect a;
    a.x = x;
    a.y = y;
    a.w = w;
    a.h = h;
    return a;
  }
}

void ebWidget::Invalidate(void) {
  if(realwidget)
    realwidget->MarkChanged();
}

void ebWidget::InvalidateArea(void) {
  if(realwidget) {
    ((ebPage *)parent)->InvalidateRect(GetArea());
    realwidget->MarkChanged();
  }
}

void ebWidget::Clone(ebWidget *newwidget) {
  // Copy property values
  // Note: override this in child objects to copy data that isn't available through properties
  int propcount = GetPropertyCount();
  for(int i=0; i<propcount; i++) {
    CustomProperty *prop = GetProperty(i);
    if(prop) {
      if(!(prop->IsReadOnly()))
        newwidget->SetPropertyValue(prop->GetName(), prop->GetValue());
    }
    else
      cerr << "ebWidget::Clone: failed to copy property #" << i << " from object " << GetName() << " of type " << GetType() << endl;
  }
}

// Property procedures

// -- x

const char *Widget_GetX(ebCustomObject *sender) {
  return inttostr(((ebWidget *)sender)->x);
}

void Widget_SetX(ebCustomObject *sender, const char *value) {
  ((ebWidget *)sender)->SetX(atoi(value));
}

// -- y

const char *Widget_GetY(ebCustomObject *sender) {
  return inttostr(((ebWidget *)sender)->y);
}

void Widget_SetY(ebCustomObject *sender, const char *value) {
  ((ebWidget *)sender)->SetY(atoi(value));
}

// -- width

const char *Widget_GetWidth(ebCustomObject *sender) {
  return inttostr(((ebWidget *)sender)->w);
}

void Widget_SetWidth(ebCustomObject *sender, const char *value) {
  ((ebWidget *)sender)->SetWidth(atoi(value));
}

// -- height

const char *Widget_GetHeight(ebCustomObject *sender) {
  return inttostr(((ebWidget *)sender)->h);
}

void Widget_SetHeight(ebCustomObject *sender, const char *value) {
  ((ebWidget *)sender)->SetHeight(atoi(value));
}

/* Method functions */

const char *Widget_SetFocus(ebCustomObject *sender, const char *method, int numargs, const char *args[]) {
  currentPage->SetFocus( ((ebWidget *)sender) );
  return "";
}
